@@ -111,8 +111,7 @@ calIcalProperty::SetValue(const nsACString &str)
         icalvalue *v = icalvalue_new_x(PromiseFlatCString(str).get());
         icalproperty_set_value(mProperty, v);
     } else if (kind == ICAL_ATTACH_VALUE) {
-        const char *strdata = PromiseFlatCString(str).get();
-        icalattach *v = icalattach_new_from_data(strdata, nullptr, nullptr);
+        icalattach *v = icalattach_new_from_data(PromiseFlatCString(str).get(), nullptr, nullptr);
         icalproperty_set_attach(mProperty, v);
     } else {
         icalproperty_set_value_from_string(mProperty,
@@ -506,14 +505,6 @@ calIcalComponent::AddTimezoneReference(calITimezone *aTimezone)
     return NS_OK;
 }
 
-static PLDHashOperator
-TimezoneHashToTimezoneArray(nsACString const& /*tzid*/, calITimezone * tz, void * arg)
-{
-    calITimezone *** const arrayPtr = static_cast<calITimezone ***>(arg);
-    NS_ADDREF(**arrayPtr = tz);
-    ++(*arrayPtr);
-    return PL_DHASH_NEXT;
-}
 
 NS_IMETHODIMP
 calIcalComponent::GetReferencedTimezones(uint32_t * aCount, calITimezone *** aTimezones)
@@ -529,11 +520,14 @@ calIcalComponent::GetReferencedTimezones(uint32_t * aCount, calITimezone *** aTi
     }
 
     calITimezone ** const timezones = static_cast<calITimezone **>(
-        nsMemory::Alloc(sizeof(calITimezone *) * count));
+        moz_xmalloc(sizeof(calITimezone *) * count));
     CAL_ENSURE_MEMORY(timezones);
     // tzptr will get used as an iterator by the enumerator function
     calITimezone ** tzptr = timezones;
-    mReferencedTimezones.EnumerateRead(TimezoneHashToTimezoneArray, &tzptr);
+    for (auto iter = mReferencedTimezones.ConstIter(); !iter.Done(); iter.Next() ) {
+        NS_ADDREF(*tzptr = iter.Data());
+        ++tzptr;
+    }
 
     *aTimezones = timezones;
     *aCount = count;
@@ -977,17 +971,6 @@ void calIcalComponent::ClearAllProperties(icalproperty_kind kind)
     }
 }
 
-static PLDHashOperator
-AddTimezoneComponentToIcal(nsACString const& /*tzid*/, calITimezone * tz, void * arg)
-{
-    icalcomponent * const comp = static_cast<icalcomponent *>(arg);
-    icaltimezone * icaltz = cal::getIcalTimezone(tz);
-    if (icaltz) {
-        icalcomponent * const tzcomp = icalcomponent_new_clone(icaltimezone_get_component(icaltz));
-        icalcomponent_add_component(comp, tzcomp);
-    }
-    return PL_DHASH_NEXT;
-}
 
 NS_IMETHODIMP
 calIcalComponent::SerializeToICS(nsACString &serialized)
@@ -1037,7 +1020,13 @@ calIcalComponent::Serialize(char **icalstr)
 
     // add the timezone bits
     if (icalcomponent_isa(mComponent) == ICAL_VCALENDAR_COMPONENT && mReferencedTimezones.Count() > 0) {
-        mReferencedTimezones.EnumerateRead(AddTimezoneComponentToIcal, mComponent);
+        for (auto iter = mReferencedTimezones.ConstIter(); !iter.Done(); iter.Next() ) {
+            icaltimezone * icaltz = cal::getIcalTimezone(iter.Data());
+            if (icaltz) {
+                icalcomponent * const tzcomp = icalcomponent_new_clone(icaltimezone_get_component(icaltz));
+                icalcomponent_add_component(mComponent, tzcomp);
+            }
+        }
     }
 
     *icalstr = icalcomponent_as_ical_string(mComponent);
@@ -1109,7 +1098,7 @@ calIcalComponent::AddSubcomponent(calIIcalComponent *aComp)
         NS_RELEASE(timezones[i]);
     }
 
-    nsMemory::Free(timezones);
+    free(timezones);
 
     if (failed)
         return rv;
@@ -1287,8 +1276,7 @@ calICSService::ParseICS(const nsACString& serialized,
 NS_IMETHODIMP
 calICSService::ParserWorker::Run()
 {
-    icalcomponent *ical =
-        icalparser_parse_string(PromiseFlatCString(mString).get());
+    icalcomponent *ical = icalparser_parse_string(mString.get());
     nsresult status = NS_OK;
     calIIcalComponent *comp = nullptr;
 
