@@ -6,6 +6,7 @@ Components.utils.import("resource://calendar/modules/calUtils.jsm");
 Components.utils.import("resource://calendar/modules/calAlarmUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/Preferences.jsm");
 
 var itemConversion = {
 
@@ -60,13 +61,13 @@ var itemConversion = {
         const copyProps = ["SUMMARY", "LOCATION", "DESCRIPTION",
                            "URL", "CLASS", "PRIORITY"];
 
-        for each (var prop in copyProps) {
+        for (var prop of copyProps) {
             aTarget.setProperty(prop, aItem.getProperty(prop));
         }
 
         // Attendees
         var attendees = aItem.getAttendees({});
-        for each (var attendee in attendees) {
+        for (var attendee of attendees) {
             aTarget.addAttendee(attendee.clone());
         }
 
@@ -106,7 +107,7 @@ var itemConversion = {
             item.dueDate = aEvent.endDate.clone();
 
             // Alarms
-            for each (let alarm in aEvent.getAlarms({})) {
+            for (let alarm of aEvent.getAlarms({})) {
                 item.addAlarm(alarm.clone());
             }
             item.alarmLastAck = (aEvent.alarmLastAck ?
@@ -144,7 +145,7 @@ var itemConversion = {
         if (!item.startDate) {
             if (aTask.dueDate) {
                 item.startDate = aTask.dueDate.clone();
-                item.startDate.minute -= getPrefSafe("calendar.event.defaultlength", 60);
+                item.startDate.minute -= Preferences.get("calendar.event.defaultlength", 60);
             } else {
                 item.startDate = cal.getDefaultStartDate();
             }
@@ -155,11 +156,11 @@ var itemConversion = {
             // Make the event be the default event length if no due date was
             // specified.
             item.endDate = item.startDate.clone();
-            item.endDate.minute += getPrefSafe("calendar.event.defaultlength", 60);
+            item.endDate.minute += Preferences.get("calendar.event.defaultlength", 60);
         }
 
         // Alarms
-        for each (let alarm in aTask.getAlarms({})) {
+        for (let alarm of aTask.getAlarms({})) {
             item.addAlarm(alarm.clone());
         }
         item.alarmLastAck = (aTask.alarmLastAck ?
@@ -232,7 +233,7 @@ calDNDBaseObserver.prototype = {
         }
 
         // Treat unicode data with VEVENT in it as text/calendar
-        if (bestFlavor.value == "text/unicode" && data.toString().indexOf("VEVENT") != -1) {
+        if (bestFlavor.value == "text/unicode" && data.toString().includes("VEVENT")) {
             bestFlavor.value = "text/calendar";
         }
 
@@ -283,7 +284,12 @@ calDNDBaseObserver.prototype = {
                 var uri = Services.io.newURI(data.toString(), null, null);
                 var loader = Components.classes["@mozilla.org/network/unichar-stream-loader;1"]
                              .createInstance(Components.interfaces.nsIUnicharStreamLoader);
-                var channel = Services.io.newChannelFromURI(uri);
+                var channel = Services.io.newChannelFromURI2(uri,
+                                                             null,
+                                                             Services.scriptSecurityManager.getSystemPrincipal(),
+                                                             null,
+                                                             Components.interfaces.nsILoadInfo.SEC_NORMAL,
+                                                             Components.interfaces.nsIContentPolicy.TYPE_OTHER);
                 channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE;
 
                 var self = this;
@@ -376,7 +382,7 @@ calViewDNDObserver.prototype = {
         var destCal = getSelectedCalendar();
         startBatchTransaction();
         try {
-            for each (var item in aItems) {
+            for (var item of aItems) {
                 doTransaction('add', item, destCal, null, null);
             }
         }
@@ -410,31 +416,10 @@ calMailButtonDNDObserver.prototype = {
      */
     onDropItems: function(aItems) {
         if (aItems && aItems.length > 0) {
-            var item = aItems[0];
-
-            var recipients = "";
-            var attendees = item.getAttendees({});
-            for each (var attendee in attendees) {
-                if (attendee.id && attendee.id.length) {
-                    var email = attendee.id;
-                    var re = new RegExp("^mailto:(.*)", "i");
-                    if (email && email.length) {
-                        if (re.test(email)) {
-                            email = RegExp.$1;
-                        } else {
-                            email = email;
-                        }
-                    }
-                    // Prevent trailing commas.
-                    if (recipients.length > 0) {
-                        recipients += ",";
-                    }
-                    // Add this recipient id to the list.
-                    recipients += email;
-                }
-            }
-
-            sendMailTo(recipients, item.title, item.getProperty("DESCRIPTION"));
+            let item = aItems[0];
+            let recipients = cal.getRecipientList(item.getAttendees({}));
+            let identity = item.calendar.getProperty("imip.identity");
+            sendMailTo(recipients, item.title, item.getProperty("DESCRIPTION"), identity);
         }
     },
 
@@ -473,7 +458,7 @@ calCalendarButtonDNDObserver.prototype = {
      * @param aItems        An array of items to handle.
      */
     onDropItems: function(aItems) {
-        for each (var item in aItems) {
+        for (var item of aItems) {
             var newItem = item;
             if (isToDo(item)) {
                 newItem = itemConversion.eventFromTask(item);
@@ -522,7 +507,7 @@ calTaskButtonDNDObserver.prototype = {
      * @param aItems        An array of items to handle.
      */
     onDropItems: function(aItems) {
-        for each (var item in aItems) {
+        for (var item of aItems) {
             var newItem = item;
             if (isEvent(item)) {
                 newItem = itemConversion.taskFromEvent(item);
