@@ -2,6 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* exported gCalendarEventTreeClicked, unifinderDoubleClick, unifinderKeyPress,
+ *          focusSearch, toggleUnifinder
+ */
+
 /**
  * U N I F I N D E R
  *
@@ -59,14 +63,14 @@ var unifinderObserver = {
     ]),
 
     // calIObserver:
-    onStartBatch: function uO_onStartBatch() {
+    onStartBatch: function() {
     },
 
-    onEndBatch: function uO_onEndBatch() {
+    onEndBatch: function() {
         refreshEventTree();
     },
 
-    onLoad: function uO_onLoad() {
+    onLoad: function() {
         if (isUnifinderHidden() && !gUnifinderNeedsRefresh) {
             // If the unifinder is hidden, all further item operations might
             // produce invalid entries in the unifinder. From now on, ignore
@@ -77,7 +81,7 @@ var unifinderObserver = {
         }
     },
 
-    onAddItem: function uO_onAddItem(aItem) {
+    onAddItem: function(aItem) {
         if (isEvent(aItem) &&
             !gUnifinderNeedsRefresh &&
             unifinderTreeView.mFilter.isItemInFilters(aItem)
@@ -86,20 +90,20 @@ var unifinderObserver = {
         }
     },
 
-    onModifyItem: function uO_onModifyItem(aNewItem, aOldItem) {
+    onModifyItem: function(aNewItem, aOldItem) {
         this.onDeleteItem(aOldItem);
         this.onAddItem(aNewItem);
     },
 
-    onDeleteItem: function uO_onDeleteItem(aDeletedItem) {
+    onDeleteItem: function(aDeletedItem) {
         if (isEvent(aDeletedItem) && !gUnifinderNeedsRefresh) {
             this.removeItemFromTree(aDeletedItem);
         }
     },
 
-    onError: function uO_onError(aCalendar, aErrNo, aMessage) {},
+    onError: function(aCalendar, aErrNo, aMessage) {},
 
-    onPropertyChanged: function uO_onPropertyChanged(aCalendar, aName, aValue, aOldValue) {
+    onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {
         switch (aName) {
             case "disabled":
                 refreshEventTree();
@@ -107,25 +111,25 @@ var unifinderObserver = {
         }
     },
 
-    onPropertyDeleting: function uO_onPropertyDeleting(aCalendar, aName) {
-      this.onPropertyChanged(aCalendar, aName, null, null);
+    onPropertyDeleting: function(aCalendar, aName) {
+        this.onPropertyChanged(aCalendar, aName, null, null);
     },
 
     // calICompositeObserver:
-    onCalendarAdded: function uO_onCalendarAdded(aAddedCalendar) {
+    onCalendarAdded: function(aAddedCalendar) {
         if (!aAddedCalendar.getProperty("disabled")) {
             addItemsFromCalendar(aAddedCalendar,
                                  addItemsFromSingleCalendarInternal);
         }
     },
 
-    onCalendarRemoved: function uO_onCalendarRemoved(aDeletedCalendar) {
+    onCalendarRemoved: function(aDeletedCalendar) {
         if (!aDeletedCalendar.getProperty("disabled")) {
             deleteItemsFromCalendar(aDeletedCalendar);
         }
     },
 
-    onDefaultCalendarChanged: function uO_onDefaultCalendarChanged(aNewDefaultCalendar) {},
+    onDefaultCalendarChanged: function(aNewDefaultCalendar) {},
 
     /**
      * Add an unifinder item to the tree. It is safe to call these for any
@@ -134,7 +138,7 @@ var unifinderObserver = {
      *
      * @return aItem        The item to add to the tree.
      */
-    addItemToTree: function uO_addItemToTree(aItem) {
+    addItemToTree: function(aItem) {
         let items;
         let filter = unifinderTreeView.mFilter;
 
@@ -153,7 +157,7 @@ var unifinderObserver = {
      *
      * @return aItem        The item to remove from the tree.
      */
-    removeItemFromTree: function uO_removeItemFromTree(aItem) {
+    removeItemFromTree: function(aItem) {
         let items;
         let filter = unifinderTreeView.mFilter;
         if (filter.startDate && filter.endDate && (aItem.parentItem == aItem)) {
@@ -165,7 +169,7 @@ var unifinderObserver = {
         unifinderTreeView.removeItems(items.filter(filter.isItemInFilters, filter));
     },
 
-    observe: function uO_observe(aSubject, aTopic, aPrefName) {
+    observe: function(aSubject, aTopic, aPrefName) {
         switch (aPrefName) {
             case "calendar.date.format":
             case "calendar.timezone.local":
@@ -257,8 +261,8 @@ function finishCalendarUnifinder() {
     let unifinderTree = document.getElementById("unifinder-search-results-tree");
     let sorted = unifinderTreeView.selectedColumn;
     if (sorted) {
-        unifinderTree.setAttribute("sort-active",sorted.getAttribute("itemproperty"));
-        unifinderTree.setAttribute("sort-direction",unifinderTreeView.sortDirection);
+        unifinderTree.setAttribute("sort-active", sorted.getAttribute("itemproperty"));
+        unifinderTree.setAttribute("sort-direction", unifinderTreeView.sortDirection);
     } else {
         unifinderTree.removeAttribute("sort-active");
         unifinderTree.removeAttribute("sort-direction");
@@ -306,7 +310,7 @@ function unifinderDoubleClick(event) {
     // find event by id
     let calendarEvent = unifinderTreeView.getItemFromEvent(event);
 
-    if (calendarEvent != null) {
+    if (calendarEvent) {
         modifyEventWithDialog(calendarEvent, null, true);
     } else {
         createEventWithDialog();
@@ -332,14 +336,14 @@ function unifinderSelect(event) {
     let end = {};
     let numRanges = tree.view.selection.getRangeCount();
 
-    for (let t = 0; t < numRanges; t++) {
-        tree.view.selection.getRangeAt(t, start, end);
+    for (let range = 0; range < numRanges; range++) {
+        tree.view.selection.getRangeAt(range, start, end);
 
-        for (let v = start.value; v <= end.value; v++) {
+        for (let i = start.value; i <= end.value; i++) {
             try {
-                selectedItems.push(unifinderTreeView.getItemAt(v));
+                selectedItems.push(unifinderTreeView.getItemAt(i));
             } catch (e) {
-               WARN("Error getting Event from row: " + e + "\n");
+                WARN("Error getting Event from row: " + e + "\n");
             }
         }
     }
@@ -353,7 +357,7 @@ function unifinderSelect(event) {
     // up in a circular loop
     currentView().setSelectedItems(selectedItems.length, selectedItems, true);
     currentView().centerSelectedItems();
-    calendarController.onSelectionChanged({detail: selectedItems});
+    calendarController.onSelectionChanged({ detail: selectedItems });
     document.getElementById("unifinder-search-results-tree").focus();
 }
 
@@ -414,10 +418,10 @@ var unifinderTreeView = {
     set selectedColumn(aCol) {
         let tree = document.getElementById("unifinder-search-results-tree");
         let treecols = tree.getElementsByTagName("treecol");
-        for each (let col in Array.slice(treecols)) {
+        for (let col of treecols) {
             if (col.getAttribute("sortActive")) {
-                  col.removeAttribute("sortActive");
-                  col.removeAttribute("sortDirection");
+                col.removeAttribute("sortActive");
+                col.removeAttribute("sortDirection");
             }
             if (aCol.getAttribute("itemproperty") == col.getAttribute("itemproperty")) {
                 col.setAttribute("sortActive", "true");
@@ -440,7 +444,7 @@ var unifinderTreeView = {
      * @param aItemArray        An array of items to add.
      * @param aDontSort         If true, the items will only be appended.
      */
-    addItems: function uTV_addItems(aItemArray, aDontSort) {
+    addItems: function(aItemArray, aDontSort) {
         this.eventArray = this.eventArray.concat(aItemArray);
         let newCount = (this.eventArray.length - aItemArray.length - 1);
         this.tree.rowCountChanged(newCount, aItemArray.length);
@@ -457,14 +461,14 @@ var unifinderTreeView = {
      *
      * @param aItemArray        An array of items to remove.
      */
-    removeItems: function uTV_removeItems(aItemArray) {
+    removeItems: function(aItemArray) {
         let indexesToRemove = [];
         // Removing items is a bit tricky. Our getItemRow function takes the
         // index from a cached map, so removing an item from the array will
         // remove the wrong indexes. We don't want to just invalidate the map,
         // since this will cause O(n^2) behavior. Instead, we keep a sorted
         // array of the indexes to remove:
-        for each (let item in aItemArray) {
+        for (let item of aItemArray) {
             let row = this.getItemRow(item);
             if (row > -1) {
                 if (!indexesToRemove.length || row <= indexesToRemove[0]) {
@@ -496,7 +500,7 @@ var unifinderTreeView = {
     /**
      * Clear all items from the unifinder.
      */
-    clearItems: function uTV_clearItems() {
+    clearItems: function() {
         let oldCount = this.eventArray.length;
         this.eventArray = [];
         this.tree.rowCountChanged(0, -oldCount);
@@ -507,10 +511,10 @@ var unifinderTreeView = {
      * Sets the items that should be in the unifinder. This removes all items
      * that were previously in the unifinder.
      */
-    setItems: function uTV_setItems(aItemArray, aDontSort) {
+    setItems: function(aItemArray, aDontSort) {
         let oldCount = this.eventArray.length;
         this.eventArray = aItemArray.slice(0);
-        this.tree.rowCountChanged(oldCount - 1 , (this.eventArray.length - oldCount));
+        this.tree.rowCountChanged(oldCount - 1, this.eventArray.length - oldCount);
 
         if (aDontSort) {
             this.calculateIndexMap();
@@ -528,9 +532,9 @@ var unifinderTreeView = {
      *                                you correctly issued rowCountChanged
      *                                notices.
      */
-    calculateIndexMap: function uTV_calculateIndexMap(aDontInvalidate) {
+    calculateIndexMap: function(aDontInvalidate) {
         this.eventIndexMap = {};
-        for (let i = 0 ; i < this.eventArray.length; i++) {
+        for (let i = 0; i < this.eventArray.length; i++) {
             this.eventIndexMap[this.eventArray[i].hashId] = i;
         }
 
@@ -542,7 +546,7 @@ var unifinderTreeView = {
     /**
      * Sort the items in the unifinder by the currently selected column.
      */
-    sortItems: function uTV_sortItems() {
+    sortItems: function() {
         if (this.selectedColumn) {
             let modifier = (this.sortDirection == "descending" ? -1 : 1);
             let sortKey = unifinderTreeView.selectedColumn.getAttribute("itemproperty");
@@ -563,7 +567,7 @@ var unifinderTreeView = {
      * @param item      The item to search for.
      * @return          The row index of the passed item.
      */
-    getItemRow: function uTV_getItemRow(item) {
+    getItemRow: function(item) {
         if (this.eventIndexMap[item.hashId] === undefined) {
             return -1;
         }
@@ -576,7 +580,7 @@ var unifinderTreeView = {
      * @param item      The row index to get the item for.
      * @return          The item at the given row.
      */
-    getItemAt: function uTV_getItemAt(aRow) {
+    getItemAt: function(aRow) {
         return this.eventArray[aRow];
     },
 
@@ -586,7 +590,7 @@ var unifinderTreeView = {
      * @param event     The DOM mouse event to get the item for.
      * @return          The item under the mouse position.
      */
-    getItemFromEvent: function uTV_getItemFromEvent(event) {
+    getItemFromEvent: function(event) {
         let row = this.tree.getRowAt(event.clientX, event.clientY);
 
         if (row > -1) {
@@ -600,8 +604,8 @@ var unifinderTreeView = {
      *
      * @param aItemArray        An array of items to select.
      */
-    setSelectedItems: function uTV_setSelectedItems(aItemArray) {
-        if (this.doingSelection || !this.tree) {
+    setSelectedItems: function(aItemArray) {
+        if (this.doingSelection || !this.tree || !this.tree.view) {
             return;
         }
 
@@ -628,26 +632,26 @@ var unifinderTreeView = {
             // If only one item is selected, scroll to it
             let rowToScrollTo = this.getItemRow(aItemArray[0]);
             if (rowToScrollTo > -1) {
-               this.tree.ensureRowIsVisible(rowToScrollTo);
-               this.tree.view.selection.select(rowToScrollTo);
+                this.tree.ensureRowIsVisible(rowToScrollTo);
+                this.tree.view.selection.select(rowToScrollTo);
             }
         } else if (aItemArray && aItemArray.length > 1) {
             // If there is more than one item, just select them all.
-            for each (let item in aItemArray) {
+            for (let item of aItemArray) {
                 let row = this.getItemRow(item);
                 this.tree.view.selection.rangedSelect(row, row, true);
             }
         }
 
         // This needs to be in a setTimeout
-        setTimeout(function() { unifinderTreeView.resetAllowSelection(); }, 1);
+        setTimeout(() => unifinderTreeView.resetAllowSelection(), 1);
     },
 
     /**
      * Due to a selection issue described in bug 168211 this method is needed to
      * re-add the selection listeners selection listeners.
      */
-    resetAllowSelection: function uTV_resetAllowSelection() {
+    resetAllowSelection: function() {
         if (!this.tree) {
             return;
         }
@@ -674,12 +678,12 @@ var unifinderTreeView = {
     // an itemTreeView that these tree views can inherit, that contains this
     // code, and possibly other code related to sorting and storing items. See
     // bug 432582 for more details.
-    getCellProperties: function uTV_getCellProperties(aRow, aCol) {
+    getCellProperties: function(aRow, aCol) {
         let rowProps = this.getRowProperties(aRow);
         let colProps = this.getColumnProperties(aCol);
         return rowProps + (rowProps && colProps ? " " : "") + colProps;
     },
-    getRowProperties: function uTV_getRowProperties(aRow) {
+    getRowProperties: function(aRow) {
         let properties = [];
         let item = this.eventArray[aRow];
         if (item.priority > 0 && item.priority < 5) {
@@ -707,122 +711,122 @@ var unifinderTreeView = {
 
         return properties.join(" ");
     },
-    getColumnProperties: function uTV_getColumnProperties(aCol) "",
+    getColumnProperties: function(aCol) { return ""; },
 
-    isContainer: function uTV_isContainer() {
+    isContainer: function() {
         return false;
     },
 
-    isContainerOpen: function uTV_isContainerOpen(aRow) {
+    isContainerOpen: function(aRow) {
         return false;
     },
 
-    isContainerEmpty: function uTV_isContainerEmpty(aRow) {
+    isContainerEmpty: function(aRow) {
         return false;
     },
 
-    isSeparator: function uTV_isSeparator(aRow) {
+    isSeparator: function(aRow) {
         return false;
     },
 
-    isSorted: function uTV_isSorted(aRow) {
+    isSorted: function(aRow) {
         return false;
     },
 
-    canDrop: function uTV_canDrop(aRow, aOrientation) {
+    canDrop: function(aRow, aOrientation) {
         return false;
     },
 
-    drop: function uTV_drop(aRow, aOrientation) {},
+    drop: function(aRow, aOrientation) {},
 
-    getParentIndex: function uTV_getParentIndex(aRow) {
+    getParentIndex: function(aRow) {
         return -1;
     },
 
-    hasNextSibling: function uTV_hasNextSibling(aRow, aAfterIndex) {},
+    hasNextSibling: function(aRow, aAfterIndex) {},
 
-    getLevel: function uTV_getLevel(aRow) {
+    getLevel: function(aRow) {
         return 0;
     },
 
-    getImageSrc: function uTV_getImageSrc(aRow, aOrientation) {},
+    getImageSrc: function(aRow, aOrientation) {},
 
-    getProgressMode: function uTV_getProgressMode(aRow, aCol) {},
+    getProgressMode: function(aRow, aCol) {},
 
-    getCellValue: function uTV_getCellValue(aRow, aCol) {
+    getCellValue: function(aRow, aCol) {
         return null;
     },
 
-    getCellText: function uTV_getCellText(row, column) {
+    getCellText: function(row, column) {
         let calendarEvent = this.eventArray[row];
 
         switch (column.element.getAttribute("itemproperty")) {
-            case "title":
-                return (calendarEvent.title ? calendarEvent.title.replace(/\n/g, ' ') : "");
-            case "startDate":
+            case "title": {
+                return (calendarEvent.title ? calendarEvent.title.replace(/\n/g, " ") : "");
+            }
+            case "startDate": {
                 return formatUnifinderEventDateTime(calendarEvent.startDate);
-
-            case "endDate":
+            }
+            case "endDate": {
                 let eventEndDate = calendarEvent.endDate.clone();
                 // XXX reimplement
-                //let eventEndDate = getCurrentNextOrPreviousRecurrence(calendarEvent);
+                // let eventEndDate = getCurrentNextOrPreviousRecurrence(calendarEvent);
                 if (calendarEvent.startDate.isDate) {
                     // display enddate is ical enddate - 1
                     eventEndDate.day = eventEndDate.day - 1;
                 }
                 return formatUnifinderEventDateTime(eventEndDate);
-
-            case "categories":
+            }
+            case "categories": {
                 return calendarEvent.getCategories({}).join(", ");
-
-            case "location":
+            }
+            case "location": {
                 return calendarEvent.getProperty("LOCATION");
-
-            case "status":
+            }
+            case "status": {
                 return getEventStatusString(calendarEvent);
-
-            case "calendar":
+            }
+            case "calendar": {
                 return calendarEvent.calendar.name;
-
-            default:
+            }
+            default: {
                 return false;
+            }
         }
     },
 
-    setTree: function uTV_setTree(tree) {
+    setTree: function(tree) {
         this.tree = tree;
     },
 
-    toggleOpenState: function uTV_toggleOpenState(aRow) {},
+    toggleOpenState: function(aRow) {},
 
-    cycleHeader: function uTV_cycleHeader(col) {
+    cycleHeader: function(col) {
         if (!this.selectedColumn) {
             this.sortDirection = "ascending";
+        } else if (!this.sortDirection || this.sortDirection == "descending") {
+            this.sortDirection = "ascending";
         } else {
-            if (!this.sortDirection || this.sortDirection == "descending") {
-                this.sortDirection = "ascending";
-            } else {
-                this.sortDirection = "descending";
-            }
+            this.sortDirection = "descending";
         }
         this.selectedColumn = col.element;
         this.sortItems();
     },
 
-    isEditable: function uTV_isEditable(aRow, aCol) {
+    isEditable: function(aRow, aCol) {
         return false;
     },
 
-    setCellValue: function uTV_setCellValue(aRow, aCol, aValue) {},
-    setCellText: function uTV_setCellText(aRow, aCol, aValue) {},
+    setCellValue: function(aRow, aCol, aValue) {},
+    setCellText: function(aRow, aCol, aValue) {},
 
-    performAction: function uTV_performAction(aAction) {},
+    performAction: function(aAction) {},
 
-    performActionOnRow: function uTV_performActionOnRow(aAction, aRow) {},
+    performActionOnRow: function(aAction, aRow) {},
 
-    performActionOnCell: function uTV_performActionOnCell(aAction, aRow, aCol) {},
+    performActionOnCell: function(aAction, aRow, aCol) {},
 
-    outParameter: new Object() // used to obtain dates during sort
+    outParameter: {} // used to obtain dates during sort
 };
 
 /**
@@ -851,9 +855,8 @@ function refreshEventTree() {
  * @param eventArray        The array of items to be set in the unifinder.
  */
 function addItemsFromCompositeCalendarInternal(eventArray) {
-    let newItems
-        = eventArray.filter(unifinderTreeView.mFilter.isItemInFilters,
-                            unifinderTreeView.mFilter);
+    let newItems = eventArray.filter(unifinderTreeView.mFilter.isItemInFilters,
+                                     unifinderTreeView.mFilter);
     unifinderTreeView.setItems(newItems);
 
     // Select selected events in the tree. Not passing the argument gets the
@@ -862,9 +865,8 @@ function addItemsFromCompositeCalendarInternal(eventArray) {
 }
 
 function addItemsFromSingleCalendarInternal(eventArray) {
-    let newItems
-        = eventArray.filter(unifinderTreeView.mFilter.isItemInFilters,
-                            unifinderTreeView.mFilter);
+    let newItems = eventArray.filter(unifinderTreeView.mFilter.isItemInFilters,
+                                     unifinderTreeView.mFilter);
     unifinderTreeView.setItems(unifinderTreeView.eventArray.concat(newItems));
 
     // Select selected events in the tree. Not passing the argument gets the
@@ -878,26 +880,18 @@ function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
         // getItems calls.
         return;
     }
-    var refreshListener = {
+    let refreshListener = {
+        QueryInterface: XPCOMUtils.generateQI([Components.interfaces.calIOperationListener]),
         mEventArray: [],
 
-        onOperationComplete: function rET_onOperationComplete(aCalendar,
-                                                              aStatus,
-                                                              aOperationType,
-                                                              aId,
-                                                              aDateTime) {
-            var refreshTreeInternalFunc = function() {
+        onOperationComplete: function(aOpCalendar, aStatus, aOperationType, aId, aDateTime) {
+            let refreshTreeInternalFunc = function() {
                 aAddItemsInternalFunc(refreshListener.mEventArray);
             };
             setTimeout(refreshTreeInternalFunc, 0);
         },
 
-        onGetResult: function rET_onGetResult(aCalendar,
-                                              aStatus,
-                                              aItemType,
-                                              aDetail,
-                                              aCount,
-                                              aItems) {
+        onGetResult: function(aOpCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
             refreshListener.mEventArray = refreshListener.mEventArray.concat(aItems);
         }
     };
@@ -925,8 +919,7 @@ function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
 
 function deleteItemsFromCalendar(aCalendar) {
     let filter = unifinderTreeView.mFilter;
-    let items = [ item for each (item in unifinderTreeView.eventArray)
-                    if (item.calendar.id == aCalendar.id) ];
+    let items = unifinderTreeView.eventArray.filter(item => item.calendar.id == aCalendar.id);
 
     unifinderTreeView.removeItems(items.filter(filter.isItemInFilters, filter));
 }
@@ -943,8 +936,8 @@ function focusSearch() {
  */
 function toggleUnifinder() {
     // Toggle the elements
-    goToggleToolbar('bottom-events-box', 'calendar_show_unifinder_command');
-    goToggleToolbar('calendar-view-splitter');
+    goToggleToolbar("bottom-events-box", "calendar_show_unifinder_command");
+    goToggleToolbar("calendar-view-splitter");
 
     unifinderTreeView.treeElement.view = unifinderTreeView;
 
