@@ -6,6 +6,7 @@ Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/AddonManager.jsm");
 Components.utils.import("resource:///modules/StringBundle.js");
+Components.utils.import("resource://gre/modules/BrowserUtils.jsm");
 
 function tabProgressListener(aTab, aStartsBlank) {
   this.mTab = aTab;
@@ -157,7 +158,7 @@ tabProgressListener.prototype =
                                          Components.interfaces.nsISupportsWeakReference])
 };
 
-const DOMLinkHandler = {
+var DOMLinkHandler = {
   handleEvent: function (event) {
     switch (event.type) {
     case "DOMLinkAdded":
@@ -171,7 +172,7 @@ const DOMLinkHandler = {
     if (!link || !link.ownerDocument || !rel || !link.href)
       return;
 
-    if (rel.split(/\s+/).indexOf("icon") != -1) {
+    if (rel.split(/\s+/).includes("icon")) {
       if (!Services.prefs.getBoolPref("browser.chrome.site_icons"))
         return;
 
@@ -185,22 +186,18 @@ const DOMLinkHandler = {
       // Verify that the load of this icon is legal.
       // Some error or special pages can load their favicon.
       // To be on the safe side, only allow chrome:// favicons.
-      let isAllowedPage = [
-        /^about:neterror\?/,
-        /^about:blocked\?/,
-        /^about:certerror\?/,
-        /^about:home$/
-      ].some(function (re) { re.test(targetDoc.documentURI); });
+      let isAllowedPage = (targetDoc.documentURI == "about:home") ||
+        ["about:neterror?",
+         "about:blocked?",
+         "about:certerror?"
+        ].some(function (aStart) { targetDoc.documentURI.startsWith(aStart); });
 
       if (!isAllowedPage || !uri.schemeIs("chrome")) {
         // Be extra paraniod and just make sure we're not going to load
         // something we shouldn't. Firefox does this, so we're doing the same.
-        let ssm = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
-          .getService(Components.interfaces.nsIScriptSecurityManager);
-
           try {
-            ssm.checkLoadURIWithPrincipal(targetDoc.nodePrincipal, uri,
-                                          Components.interfaces.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+            Services.scriptSecurityManager.checkLoadURIWithPrincipal(targetDoc.nodePrincipal, uri,
+                                           Components.interfaces.nsIScriptSecurityManager.DISALLOW_SCRIPT);
           }
           catch (ex) {
             return;
@@ -242,12 +239,15 @@ const DOMLinkHandler = {
   }
 };
 
-const kTelemetryPrompted    = "toolkit.telemetry.prompted";
-const kTelemetryEnabled     = "toolkit.telemetry.enabled";
-const kTelemetryServerOwner = "toolkit.telemetry.server_owner";
+var kTelemetryPrompted    = "toolkit.telemetry.prompted";
+var kTelemetryEnabled     = "toolkit.telemetry.enabled";
+var kTelemetryRejected    = "toolkit.telemetry.rejected";
+var kTelemetryServerOwner = "toolkit.telemetry.server_owner";
+// This is used to reprompt/renotify users when privacy message changes
+var kTelemetryPromptRev   = 2;
 
 var contentTabBaseType = {
-  inContentWhitelist: ['about:addons'],
+  inContentWhitelist: ['about:addons', 'about:preferences'],
   shouldSwitchTo: function onSwitchTo({contentPage: aContentPage}) {
     let tabmail = document.getElementById("tabmail");
     let tabInfo = tabmail.tabInfo;
@@ -297,7 +297,7 @@ var contentTabBaseType = {
   },
 
   hideChromeForLocation: function hideChromeForLocation(aLocation) {
-    return this.inContentWhitelist.indexOf(aLocation) != -1;
+    return this.inContentWhitelist.includes(aLocation);
   },
 
   /* _setUpLoadListener attaches a load listener to the tab browser that
@@ -426,7 +426,8 @@ var contentTabBaseType = {
         PrintUtils.showPageSetup();
         break;
       case "cmd_print":
-        PrintUtils.print();
+        let browser = this.getBrowser(aTab);
+        PrintUtils.printWindow(browser.outerWindowID, browser);
         break;
       // XXX print preview not currently supported - bug 497994 to implement.
       //case "cmd_printpreview":
@@ -458,6 +459,105 @@ var specialTabs = {
                 .getService(Components.interfaces.nsIFaviconService);
   },
 
+  /**
+   * We use an html image node to test the favicon, errors are well returned.
+   * Returning a url for nsITreeView.getImageSrc() will not indicate any
+   * error, and setAndFetchFaviconForPage() can't be used to detect
+   * failed icons due to Bug 740457. This also ensures 301 Moved or
+   * redirected urls will work (they won't otherwise in getImageSrc).
+   *
+   * @param  function successFunc - caller's success function.
+   * @param  function errorFunc   - caller's error function.
+   * @param  string iconUrl       - url to load.
+   * @return HTMLImageElement imageNode
+   */
+  loadFaviconImageNode: function(successFunc, errorFunc, iconUrl) {
+    let HTMLNS = "http://www.w3.org/1999/xhtml";
+    let imageNode = document.createElementNS(HTMLNS, "img");
+    imageNode.style.visibility = "collapse";
+    imageNode.addEventListener("load", event => successFunc(event, iconUrl),
+                               {capture: false, once: true});
+    imageNode.addEventListener("error", event => errorFunc(event, iconUrl),
+                               {capture: false, once: true});
+    imageNode.src = iconUrl;
+    return imageNode;
+  },
+
+  /**
+   * Favicon request timeout, 20 seconds.
+   */
+  REQUEST_TIMEOUT: 20 * 1000,
+
+  /**
+   * Get the favicon by parsing for <link rel=""> with "icon" from the page's
+   * dom <head>.
+   *
+   * @param  string aUrl          - a url from whose homepage to get a favicon.
+   * @param  function aCallback   - callback.
+   */
+  getFaviconFromPage: function(aUrl, aCallback) {
+    let url, uri;
+    try {
+      url = Services.io.newURI(aUrl, null, null).prePath;
+      uri = Services.io.newURI(url, null, null);
+    }
+    catch (ex) {
+      if (aCallback)
+        aCallback("");
+      return;
+    }
+
+    let onLoadSuccess = (aEvent => {
+      let iconUri = Services.io.newURI(aEvent.target.src, null, null);
+      specialTabs.mFaviconService.setAndFetchFaviconForPage(
+        uri, iconUri, false,
+        specialTabs.mFaviconService.FAVICON_LOAD_NON_PRIVATE,
+        null, Services.scriptSecurityManager.getSystemPrincipal());
+
+      if (aCallback)
+        aCallback(iconUri.spec);
+    });
+
+    let onDownloadError = (aEvent => {
+      if (aCallback)
+        aCallback("");
+    });
+
+    let onDownload = (aEvent => {
+      let request = aEvent.target;
+      let dom = request.response;
+      if (request.status != 200 || !(dom instanceof Ci.nsIDOMHTMLDocument)) {
+        onDownloadError(aEvent);
+        return;
+      }
+
+      let iconUri;
+      let linkNode = dom.head.querySelector('link[rel="shortcut icon"],' +
+                                            'link[rel="icon"]');
+      let href = linkNode ? linkNode.href : null;
+      try {
+        iconUri = Services.io.newURI(href, null, null);
+      }
+      catch (ex) {
+        onDownloadError(aEvent);
+        return;
+      }
+
+      specialTabs.loadFaviconImageNode(onLoadSuccess, onDownloadError,
+                                       iconUri.spec);
+    });
+
+    let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
+                    .createInstance(Ci.nsIXMLHttpRequest);
+    request.open("GET", url, true);
+    request.responseType = "document";
+    request.onload = onDownload;
+    request.onerror = onDownloadError;
+    request.timeout = this.REQUEST_TIMEOUT;
+    request.ontimeout = onDownloadError;
+    request.send(null);
+  },
+
   // This will open any special tabs if necessary on startup.
   openSpecialTabsOnStartup: function() {
     window.addEventListener("unload", specialTabs.onunload, false);
@@ -480,8 +580,7 @@ var specialTabs = {
 
     // enable global history
     try {
-      browser.docShell.QueryInterface(Components.interfaces.nsIDocShellHistory)
-             .useGlobalHistory = true;
+      browser.docShell.useGlobalHistory = true;
     } catch(ex) {
       Components.utils.reportError("Places database may be locked: " + ex);
     }
@@ -490,15 +589,12 @@ var specialTabs = {
 
     let tabmail = document.getElementById('tabmail');
 
-    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-                          .getService(Components.interfaces.nsIPrefBranch);
-
     tabmail.registerTabType(this.contentTabType);
     tabmail.registerTabType(this.chromeTabType);
 
     // If we've upgraded (note: always get these values so that we set
     // the mstone preference for the new version):
-    let [fromVer, toVer] = this.getApplicationUpgradeVersions(prefs);
+    let [fromVer, toVer] = this.getApplicationUpgradeVersions();
 
     // Although this might not be really necessary because of the version checks, we'll
     // check this pref anyway and clear it so that we are consistent with what Firefox
@@ -507,7 +603,7 @@ var specialTabs = {
       // Only show what's new tab if this is actually an upgraded version,
       // not just a new installation/profile (and don't show if the major version
       // hasn't changed).
-      if (fromVer && ((fromVer[0] != toVer[0]) || (fromVer[1] != toVer[1]))) {
+      if (fromVer && (fromVer[0] != toVer[0])) {
           // showWhatsNewPage checks the details of the update manager before
           // showing the page.
           this.showWhatsNewPage();
@@ -516,10 +612,10 @@ var specialTabs = {
     }
 
     // Show the about rights notification if we need to.
-    if (this.shouldShowAboutRightsNotification(prefs))
-      this.showAboutRightsNotification(prefs);
-    else if (this.shouldShowTelemetryNotification(prefs))
-      this.showTelemetryNotification(prefs);
+    if (this.shouldShowAboutRightsNotification())
+      this.showAboutRightsNotification();
+    else if (this.shouldShowTelemetryNotification())
+      this.showTelemetryNotification();
   },
 
   /**
@@ -575,8 +671,8 @@ var specialTabs = {
       aTab.root = clone;
 
       // Start setting up the browser.
-      aTab.browser = aTab.panel.getElementsByTagName("browser")[0];
-      aTab.toolbar = aTab.panel.getElementsByClassName("contentTabToolbar")[0];
+      aTab.browser = aTab.panel.querySelector("browser");
+      aTab.toolbar = aTab.panel.querySelector(".contentTabToolbar");
 
       // As we're opening this tab, showTab may not get called, so set
       // the type according to if we're opening in background or not.
@@ -599,7 +695,7 @@ var specialTabs = {
       gPluginHandler.addEventListeners(aTab.browser);
 
       // Now initialise the find bar.
-      aTab.findbar = aTab.panel.getElementsByTagName("findbar")[0];
+      aTab.findbar = aTab.panel.querySelector("findbar");
       aTab.findbar.setAttribute("browserid",
                                 "contentTabBrowser" + this.lastBrowserId);
 
@@ -694,21 +790,21 @@ var specialTabs = {
    * or the pref being set to ignore - return null and the current version.
    * In either case, updates the pref with the latest version.
    */
-  getApplicationUpgradeVersions: function(prefs) {
+  getApplicationUpgradeVersions: function() {
     let savedAppVersion = null;
     let prefstring = "mailnews.start_page_override.mstone";
 
     try {
-      savedAppVersion = prefs.getCharPref(prefstring);
+      savedAppVersion = Services.prefs.getCharPref(prefstring);
     } catch (ex) {}
 
-    let currentApplicationVersion = Application.version;
+    let currentApplicationVersion = Services.appinfo.version;
 
     if (savedAppVersion == "ignore")
       return [null, this.splitVersion(currentApplicationVersion)];
 
     if (savedAppVersion != currentApplicationVersion)
-      prefs.setCharPref(prefstring, currentApplicationVersion);
+      Services.prefs.setCharPref(prefstring, currentApplicationVersion);
 
     return [this.splitVersion(savedAppVersion), this.splitVersion(currentApplicationVersion)];
   },
@@ -730,7 +826,7 @@ var specialTabs = {
     }
 
     let actions = update.getProperty("actions");
-    if (actions && actions.indexOf("silent") != -1)
+    if (actions && actions.includes("silent"))
       return;
 
     openWhatsNew();
@@ -742,16 +838,31 @@ var specialTabs = {
    *
    * This is controlled by the pref toolkit.telemetry.prompted
    */
-  shouldShowTelemetryNotification: function(prefs) {
-    // toolkit has decided that the pref should have no default value
+  shouldShowTelemetryNotification: function() {
+    // Toolkit has decided that the pref should have no default value, so this
+    // throws if not yet initialized.
+    let telemetryPrompted = false;
     try {
-      if (prefs.getBoolPref(kTelemetryPrompted) || prefs.getBoolPref(kTelemetryEnabled))
-        return false;
+      telemetryPrompted = (Services.prefs.getIntPref(kTelemetryPrompted) >= kTelemetryPromptRev);
     } catch (e) { }
+    let telemetryEnabled = false;
+    try {
+      telemetryEnabled = (Services.prefs.getBoolPref(kTelemetryEnabled));
+    } catch (e) { }
+    // In case user already allowed telemetry, do not bother him with any updated
+    // prompt. Clear the pref first, in case it was not Int (from older versions).
+    if (telemetryEnabled && !telemetryPrompted) {
+      Services.prefs.clearUserPref(kTelemetryPrompted);
+      Services.prefs.setIntPref(kTelemetryPrompted, kTelemetryPromptRev);
+    }
+
+    if (telemetryEnabled || telemetryPrompted)
+      return false;
+
     return true;
   },
 
-  showTelemetryNotification: function(prefs) {
+  showTelemetryNotification: function() {
     var notifyBox = document.getElementById("mail-notification-box");
 
     var brandBundle =
@@ -760,28 +871,35 @@ var specialTabs = {
       new StringBundle("chrome://messenger/locale/telemetry.properties");
 
     var productName = brandBundle.get("brandFullName");
-    var serverOwner = prefs.getCharPref(kTelemetryServerOwner);
+    var serverOwner = Services.prefs.getCharPref(kTelemetryServerOwner);
     var telemetryText = telemetryBundle.get("telemetryText", [productName, serverOwner]);
 
+    // Clear all the prefs as we will set them as needed after answering the prompt.
+    Services.prefs.clearUserPref(kTelemetryPrompted);
+    Services.prefs.clearUserPref(kTelemetryEnabled);
+    Services.prefs.clearUserPref(kTelemetryRejected);
+
     var buttons = [
       {
         label:     telemetryBundle.get("telemetryYesButtonLabel"),
         accessKey: telemetryBundle.get("telemetryYesButtonAccessKey"),
         popup:     null,
         callback:  function(aNotificationBar, aButton) {
-          prefs.setBoolPref(kTelemetryEnabled, true);
+          Services.prefs.setBoolPref(kTelemetryEnabled, true);
         }
       },
       {
         label:     telemetryBundle.get("telemetryNoButtonLabel"),
         accessKey: telemetryBundle.get("telemetryNoButtonAccessKey"),
         popup:     null,
-        callback:  function(aNotificationBar, aButton) {}
+        callback:  function(aNotificationBar, aButton) {
+          Services.prefs.setBoolPref(kTelemetryRejected, true);
+        }
       }
     ];
 
     // Set pref to indicate we've shown the notification.
-    prefs.setBoolPref(kTelemetryPrompted, true);
+    Services.prefs.setIntPref(kTelemetryPrompted, kTelemetryPromptRev);
 
     var notification = notifyBox.appendNotification(telemetryText, "telemetry", null, notifyBox.PRIORITY_INFO_LOW, buttons);
     notification.persistence = 3; // arbitrary number, just so bar sticks around for a bit
@@ -818,24 +936,21 @@ var specialTabs = {
    *     If this pref isn't set or the value is less than the current version
    *     then we show the about:rights notification.
    */
-  shouldShowAboutRightsNotification: function(prefs) {
+  shouldShowAboutRightsNotification: function() {
     try {
-      return !prefs.getBoolPref("mail.rights.override");
+      return !Services.prefs.getBoolPref("mail.rights.override");
     } catch (e) { }
 
-    return prefs.getIntPref("mail.rights.version") < this._kAboutRightsVersion;
+    return Services.prefs.getIntPref("mail.rights.version") < this._kAboutRightsVersion;
   },
 
-  showAboutRightsNotification: function(prefs) {
+  showAboutRightsNotification: function() {
     var notifyBox = document.getElementById("mail-notification-box");
 
-    var stringBundle =
-      Components.classes["@mozilla.org/intl/stringbundle;1"]
-                .getService(Components.interfaces.nsIStringBundleService);
     var brandBundle =
-      stringBundle.createBundle("chrome://branding/locale/brand.properties");
+      Services.strings.createBundle("chrome://branding/locale/brand.properties");
     var rightsBundle =
-      stringBundle.createBundle("chrome://messenger/locale/aboutRights.properties");
+      Services.strings.createBundle("chrome://messenger/locale/aboutRights.properties");
 
     var productName = brandBundle.GetStringFromName("brandFullName");
     var notifyRightsText = rightsBundle.formatStringFromName("notifyRightsText",
@@ -862,7 +977,7 @@ var specialTabs = {
     box.persistence = 3;
 
     // Set the pref to say we've displayed the notification.
-    prefs.setIntPref("mail.rights.version", this._kAboutRightsVersion);
+    Services.prefs.setIntPref("mail.rights.version", this._kAboutRightsVersion);
   },
 
   /**
@@ -971,7 +1086,7 @@ var specialTabs = {
       }
     },
 
-    shouldSwitchTo: function ({ chromePage: x })
+    shouldSwitchTo: ({ chromePage: x }) =>
       contentTabBaseType.shouldSwitchTo({ contentPage: x }),
 
     /**
@@ -1005,7 +1120,7 @@ var specialTabs = {
       aTab.panel.appendChild(clone);
 
       // Start setting up the browser.
-      aTab.browser = aTab.panel.getElementsByTagName("browser")[0];
+      aTab.browser = aTab.panel.querySelector("browser");
 
       // As we're opening this tab, showTab may not get called, so set
       // the type according to if we're opening in background or not.
@@ -1134,7 +1249,8 @@ var specialTabs = {
           PrintUtils.showPageSetup();
           break;
         case "cmd_print":
-          PrintUtils.print();
+          let browser = this.getBrowser(aTab);
+          PrintUtils.printWindow(browser.outerWindowID, browser);
           break;
         // XXX print preview not currently supported - bug 497994 to implement.
         //case "cmd_printpreview":
@@ -1217,8 +1333,8 @@ var specialTabs = {
       let messengerBundle = document.getElementById("bundle_messenger");
 
       let installInfo = aSubject.QueryInterface(Ci.amIWebInstallInfo);
-      let win = installInfo.originatingWindow;
-      let notificationBox = getNotificationBox(win.top);
+      let browser = installInfo.browser;
+      let notificationBox = getNotificationBox(browser.contentWindow);
       let notificationID = aTopic;
       let brandShortName = brandBundle.getString("brandShortName");
       let notificationName, messageString, buttons;
@@ -1269,7 +1385,7 @@ var specialTabs = {
         if (notificationBox && !notificationBox.getNotificationWithValue(notificationName)) {
             notificationBox.appendNotification(messageString, notificationName,
                                                iconURL,
-                                               notificationBox.PRIORITY_MEDIUM_HIGH,
+                                               notificationBox.PRIORITY_WARNING_MEDIUM,
                                                buttons);
           }
         break;
@@ -1319,7 +1435,7 @@ var specialTabs = {
             accessKey: messengerBundle.getString("addonInstallRestartButton.accesskey"),
             popup: null,
             callback: function() {
-              Application.restart();
+              BrowserUtils.restartApplication();
             }
           }];
         }
@@ -1372,8 +1488,8 @@ var specialTabs = {
    */
   _shouldLoadFavIcon: function shouldLoadFavIcon(aURI) {
     return (aURI &&
-            Application.prefs.getValue("browser.chrome.site_icons", false) &&
-            Application.prefs.getValue("browser.chrome.favicons", false) &&
+            Services.prefs.getBoolPref("browser.chrome.site_icons") &&
+            Services.prefs.getBoolPref("browser.chrome.favicons") &&
             ("schemeIs" in aURI) &&
             (aURI.schemeIs("http") || aURI.schemeIs("https")));
   },
@@ -1422,8 +1538,10 @@ var specialTabs = {
    */
   setTabIcon: function(aTab, aIcon) {
     if (aIcon && this.mFaviconService)
-      this.mFaviconService.setAndFetchFaviconForPage(aTab.browser.currentURI,
-                                                     makeURI(aIcon), false);
+      this.mFaviconService.setAndFetchFaviconForPage(
+        aTab.browser.currentURI, makeURI(aIcon), false,
+        this.mFaviconService.FAVICON_LOAD_NON_PRIVATE,
+        null, aTab.browser.contentPrincipal);
 
     // Save this off so we know about it later,
     aTab.browser.mIconURL = aIcon;
