@@ -8,8 +8,9 @@
 #include "nsCOMPtr.h"
 #include "nsMsgThread.h"
 #include "MailNewsTypes2.h"
+#include "mozilla/DebugOnly.h"
 
-NS_IMPL_ISUPPORTS1(nsMsgThread, nsIMsgThread)
+NS_IMPL_ISUPPORTS(nsMsgThread, nsIMsgThread)
 
 nsMsgThread::nsMsgThread()
 {
@@ -53,11 +54,11 @@ nsMsgThread::~nsMsgThread()
   MOZ_COUNT_DTOR(nsMsgThread);
   if (m_mdbDB)
   {
-    bool found = m_mdbDB->m_threads.RemoveElement(this);
+    mozilla::DebugOnly<bool> found = m_mdbDB->m_threads.RemoveElement(this);
     NS_ASSERTION(found, "removing thread not in threads array");
   }
-  else
-    NS_ERROR("null db in thread");
+  else // This can happen if db is forced closed
+    NS_WARNING("null db in thread");
   Clear();
 }
 
@@ -210,6 +211,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
   bool parentKeyNeedsSetting = true;
 
   nsIMdbRow *hdrRow = hdr->GetMDBRow();
+  NS_ENSURE_STATE(hdrRow);
   hdr->GetRawFlags(&newHdrFlags);
   hdr->GetMessageKey(&newHdrKey);
   hdr->GetDateInSeconds(&msgDate);
@@ -253,8 +255,8 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
     child->SetThreadParent(parentKey);
     parentKeyNeedsSetting = false;
   }
-  // check if this header is a parent of one of the messages in this thread
 
+  // check if this header is a parent of one of the messages in this thread
   bool hdrMoved = false;
   nsCOMPtr <nsIMsgDBHdr> curHdr;
   uint32_t moveIndex = 0;
@@ -276,7 +278,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
   {
     for (childIndex = 0; childIndex < numChildren; childIndex++)
     {
-      nsMsgKey msgKey;
+      nsMsgKey msgKey = nsMsgKey_None;
 
       rv = GetChildHdrAt(childIndex, getter_AddRefs(curHdr));
       if (NS_SUCCEEDED(rv) && curHdr)
@@ -312,6 +314,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
             }
           }
           curHdr->SetThreadParent(newHdrKey);
+          // TODO: what should be msgKey if hdrMoved was true above?
           if (msgKey == newHdrKey)
             parentKeyNeedsSetting = false;
 
@@ -429,12 +432,12 @@ nsresult nsMsgThread::ReparentNonReferenceChildrenOf(nsIMsgDBHdr *oldTopLevelHdr
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgThread::GetChildKeyAt(int32_t aIndex, nsMsgKey *aResult)
+NS_IMETHODIMP nsMsgThread::GetChildKeyAt(uint32_t aIndex, nsMsgKey *aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
   nsresult rv;
 
-  if (aIndex >= (int32_t) m_numChildren)
+  if (aIndex >= m_numChildren)
   {
     *aResult = nsMsgKey_None;
     return NS_ERROR_ILLEGAL_VALUE;
@@ -447,11 +450,11 @@ NS_IMETHODIMP nsMsgThread::GetChildKeyAt(int32_t aIndex, nsMsgKey *aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgThread::GetChildHdrAt(int32_t aIndex, nsIMsgDBHdr **result)
+NS_IMETHODIMP nsMsgThread::GetChildHdrAt(uint32_t aIndex, nsIMsgDBHdr **result)
 {
   // mork doesn't seem to handle this correctly, so deal with going off
   // the end here.
-  if (aIndex < 0 || uint32_t(aIndex) >= m_numChildren)
+  if (aIndex >= m_numChildren)
     return NS_MSG_MESSAGE_NOT_FOUND;
   mdbOid oid;
   nsresult rv = m_mdbTable->PosToOid( m_mdbDB->GetEnv(), aIndex, &oid);
@@ -490,7 +493,7 @@ NS_IMETHODIMP nsMsgThread::GetChild(nsMsgKey msgKey, nsIMsgDBHdr **result)
   return rv;
 }
 
-NS_IMETHODIMP nsMsgThread::RemoveChildAt(int32_t aIndex)
+NS_IMETHODIMP nsMsgThread::RemoveChildAt(uint32_t aIndex)
 {
   return NS_OK;
 }
@@ -607,9 +610,9 @@ public:
   nsMsgThreadEnumerator(nsMsgThread *thread, nsMsgKey startKey,
   nsMsgThreadEnumeratorFilter filter, void* closure);
   int32_t MsgKeyFirstChildIndex(nsMsgKey inMsgKey);
-  virtual ~nsMsgThreadEnumerator();
 
 protected:
+  virtual ~nsMsgThreadEnumerator();
 
   nsresult                Prefetch();
 
@@ -701,7 +704,7 @@ nsMsgThreadEnumerator::~nsMsgThreadEnumerator()
     NS_RELEASE(mThread);
 }
 
-NS_IMPL_ISUPPORTS1(nsMsgThreadEnumerator, nsISimpleEnumerator)
+NS_IMPL_ISUPPORTS(nsMsgThreadEnumerator, nsISimpleEnumerator)
 
 
 int32_t nsMsgThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
@@ -854,7 +857,7 @@ nsresult nsMsgThread::ReparentMsgsWithInvalidParent(uint32_t numChildren, nsMsgK
   nsresult rv = NS_OK;
   // run through looking for messages that don't have a correct parent,
   // i.e., a parent that's in the thread!
-  for (int32_t childIndex = 0; childIndex < (int32_t) numChildren; childIndex++)
+  for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
     nsCOMPtr <nsIMsgDBHdr> curChild;
     rv  = GetChildHdrAt(childIndex, getter_AddRefs(curChild));
@@ -912,7 +915,7 @@ NS_IMETHODIMP nsMsgThread::GetRootHdr(int32_t *resultIndex, nsIMsgDBHdr **result
     nsMsgKey threadParentKey = nsMsgKey_None;
     GetNumChildren(&numChildren);
 
-    for (int32_t childIndex = 0; childIndex < (int32_t) numChildren; childIndex++)
+    for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
       nsCOMPtr <nsIMsgDBHdr> curChild;
       rv  = GetChildHdrAt(childIndex, getter_AddRefs(curChild));
@@ -965,10 +968,10 @@ nsresult nsMsgThread::ChangeChildCount(int32_t delta)
   uint32_t childCount = 0;
   m_mdbDB->RowCellColumnToUInt32(m_metaRow, m_mdbDB->m_threadChildrenColumnToken, childCount);
 
-  NS_WARN_IF_FALSE(childCount != 0 || delta > 0, "child count gone negative");
+  NS_WARNING_ASSERTION(childCount != 0 || delta > 0, "child count gone negative");
   childCount += delta;
 
-  NS_WARN_IF_FALSE((int32_t) childCount >= 0, "child count gone to 0 or below");
+  NS_WARNING_ASSERTION((int32_t) childCount >= 0, "child count gone to 0 or below");
   if ((int32_t) childCount < 0)	// force child count to >= 0
     childCount = 0;
 
@@ -1073,7 +1076,7 @@ nsresult nsMsgThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **resul
     }
   }
   if (resultIndex)
-    *resultIndex = childIndex;
+    *resultIndex = (int32_t) childIndex;
 
   return rv;
 }
