@@ -9,12 +9,12 @@
 
 #include "mozilla/Monitor.h"
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(StreamListenerProxy, nsIStreamListener)
-NS_IMPL_THREADSAFE_ISUPPORTS1(ImapMailFolderSinkProxy, nsIImapMailFolderSink)
-NS_IMPL_THREADSAFE_ISUPPORTS1(ImapServerSinkProxy, nsIImapServerSink)
-NS_IMPL_THREADSAFE_ISUPPORTS1(ImapMessageSinkProxy,
+NS_IMPL_ISUPPORTS(StreamListenerProxy, nsIStreamListener)
+NS_IMPL_ISUPPORTS(ImapMailFolderSinkProxy, nsIImapMailFolderSink)
+NS_IMPL_ISUPPORTS(ImapServerSinkProxy, nsIImapServerSink)
+NS_IMPL_ISUPPORTS(ImapMessageSinkProxy,
                               nsIImapMessageSink)
-NS_IMPL_THREADSAFE_ISUPPORTS1(ImapProtocolSinkProxy,
+NS_IMPL_ISUPPORTS(ImapProtocolSinkProxy,
                               nsIImapProtocolSink)
 namespace {
 
@@ -56,7 +56,7 @@ struct RefType<const nsIID&>
   typedef const nsIID& type;
 };
 
-class SyncRunnableBase : public nsRunnable
+class SyncRunnableBase : public mozilla::Runnable
 {
 public:
   nsresult Result() {
@@ -282,7 +282,7 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 
 #define NS_SYNCRUNNABLEMETHOD0(iface, method)                       \
   NS_IMETHODIMP iface##Proxy::method() {                     \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable0<nsI##iface>                           \
       (mReceiver, &nsI##iface::method);                         \
     return DispatchSyncRunnable(r);                                 \
@@ -292,7 +292,7 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 #define NS_SYNCRUNNABLEMETHOD1(iface, method,                       \
                                arg1)                                \
   NS_IMETHODIMP iface##Proxy::method(arg1 a1) {                     \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable1<nsI##iface, arg1>                           \
       (mReceiver, &nsI##iface::method, a1);                         \
     return DispatchSyncRunnable(r);                                 \
@@ -301,7 +301,7 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 #define NS_SYNCRUNNABLEMETHOD2(iface, method,                       \
                                arg1, arg2)                          \
   NS_IMETHODIMP iface##Proxy::method(arg1 a1, arg2 a2) {            \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable2<nsI##iface, arg1, arg2>                     \
       (mReceiver, &nsI##iface::method, a1, a2);                     \
     return DispatchSyncRunnable(r);                                 \
@@ -310,7 +310,7 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 #define NS_SYNCRUNNABLEMETHOD3(iface, method,                       \
                                arg1, arg2, arg3)                    \
   NS_IMETHODIMP iface##Proxy::method(arg1 a1, arg2 a2, arg3 a3) {   \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable3<nsI##iface, arg1, arg2, arg3>               \
       (mReceiver, &nsI##iface::method,                              \
        a1, a2, a3);                                                 \
@@ -320,7 +320,7 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 #define NS_SYNCRUNNABLEMETHOD4(iface, method,                       \
                                arg1, arg2, arg3, arg4)              \
   NS_IMETHODIMP iface##Proxy::method(arg1 a1, arg2 a2, arg3 a3, arg4 a4) { \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable4<nsI##iface, arg1, arg2, arg3, arg4>         \
       (mReceiver, &nsI##iface::method,                              \
        a1, a2, a3, a4);                                             \
@@ -330,7 +330,7 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 #define NS_SYNCRUNNABLEMETHOD5(iface, method,                       \
                                arg1, arg2, arg3, arg4, arg5)        \
   NS_IMETHODIMP iface##Proxy::method(arg1 a1, arg2 a2, arg3 a3, arg4 a4, arg5 a5) {   \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable5<nsI##iface, arg1, arg2, arg3, arg4, arg5>   \
       (mReceiver, &nsI##iface::method,                              \
        a1, a2, a3, a4, a5);                                         \
@@ -340,13 +340,13 @@ DispatchSyncRunnable(SyncRunnableBase* r)
 #define NS_SYNCRUNNABLEATTRIBUTE(iface, attribute,                       \
                                  type)                                \
 NS_IMETHODIMP iface##Proxy::Get##attribute(type *a1) {                     \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable1<nsI##iface, type *>                           \
       (mReceiver, &nsI##iface::Get##attribute, a1);                         \
     return DispatchSyncRunnable(r);                                 \
   } \
 NS_IMETHODIMP iface##Proxy::Set##attribute(type a1) {                     \
-    nsRefPtr<SyncRunnableBase> r =                                  \
+    RefPtr<SyncRunnableBase> r =                                  \
       new SyncRunnable1<nsI##iface, type>                           \
       (mReceiver, &nsI##iface::Set##attribute, a1);                         \
     return DispatchSyncRunnable(r);                                 \
@@ -357,7 +357,7 @@ NS_IMETHODIMP iface##Proxy::Set##attribute(type a1) {                     \
   { NS_RUNTIMEABORT("Not implemented"); return NS_ERROR_UNEXPECTED; }
 
 NS_SYNCRUNNABLEMETHOD5(StreamListener, OnDataAvailable,
-                       nsIRequest *, nsISupports *, nsIInputStream *, PRUint32, PRUint32)
+                       nsIRequest *, nsISupports *, nsIInputStream *, uint64_t, uint32_t)
 
 NS_SYNCRUNNABLEMETHOD2(StreamListener, OnStartRequest,
                        nsIRequest *, nsISupports *)
@@ -369,20 +369,21 @@ NS_SYNCRUNNABLEMETHOD2(ImapProtocolSink, GetUrlWindow, nsIMsgMailNewsUrl *,
                        nsIMsgWindow **)
 
 NS_SYNCRUNNABLEMETHOD0(ImapProtocolSink, CloseStreams)
+NS_SYNCRUNNABLEMETHOD0(ImapProtocolSink, SetupMainThreadProxies)
 
 NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, FolderNeedsACLListed, bool)
 NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, FolderNeedsSubscribing, bool)
 NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, FolderNeedsAdded, bool)
-NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, AclFlags, PRUint32)
-NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, UidValidity, PRInt32)
+NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, AclFlags, uint32_t)
+NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, UidValidity, int32_t)
 NS_SYNCRUNNABLEATTRIBUTE(ImapMailFolderSink, FolderQuotaCommandIssued, bool)
-NS_SYNCRUNNABLEMETHOD3(ImapMailFolderSink, SetFolderQuotaData, const nsACString &, PRUint32, PRUint32)
+NS_SYNCRUNNABLEMETHOD3(ImapMailFolderSink, SetFolderQuotaData, const nsACString &, uint32_t, uint32_t)
 NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, GetShouldDownloadAllHeaders, bool *)
 NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, GetOnlineDelimiter, char *)
 NS_SYNCRUNNABLEMETHOD0(ImapMailFolderSink, OnNewIdleMessages)
 NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, UpdateImapMailboxStatus, nsIImapProtocol *, nsIMailboxSpec *)
 NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, UpdateImapMailboxInfo, nsIImapProtocol *, nsIMailboxSpec *)
-NS_SYNCRUNNABLEMETHOD4(ImapMailFolderSink, GetMsgHdrsToDownload, bool *, PRInt32 *, PRUint32 *, nsMsgKey **)
+NS_SYNCRUNNABLEMETHOD4(ImapMailFolderSink, GetMsgHdrsToDownload, bool *, int32_t *, uint32_t *, nsMsgKey **)
 NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, ParseMsgHdrs, nsIImapProtocol *, nsIImapHeaderXferInfo *)
 NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, AbortHeaderParseStream, nsIImapProtocol *)
 NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, OnlineCopyCompleted, nsIImapProtocol *, ImapOnlineCopyState)
@@ -395,9 +396,9 @@ NS_SYNCRUNNABLEMETHOD5(ImapMailFolderSink, SetUrlState, nsIImapProtocol *, nsIMs
                        bool, bool, nsresult)
 NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, ReleaseUrlCacheEntry, nsIMsgMailNewsUrl *)
 NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, HeaderFetchCompleted, nsIImapProtocol *)
-NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, SetBiffStateAndUpdate, PRInt32)
-NS_SYNCRUNNABLEMETHOD3(ImapMailFolderSink, ProgressStatus, nsIImapProtocol*, PRUint32, const PRUnichar *)
-NS_SYNCRUNNABLEMETHOD4(ImapMailFolderSink, PercentProgress, nsIImapProtocol*, const PRUnichar *, PRInt64, PRInt64)
+NS_SYNCRUNNABLEMETHOD1(ImapMailFolderSink, SetBiffStateAndUpdate, int32_t)
+NS_SYNCRUNNABLEMETHOD3(ImapMailFolderSink, ProgressStatusString, nsIImapProtocol*, const char*, const char16_t *)
+NS_SYNCRUNNABLEMETHOD4(ImapMailFolderSink, PercentProgress, nsIImapProtocol*, const char16_t *, int64_t, int64_t)
 NS_SYNCRUNNABLEMETHOD0(ImapMailFolderSink, ClearFolderRights)
 NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, SetCopyResponseUid, const char *, nsIImapUrl *)
 NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, SetAppendMsgUid, nsMsgKey, nsIImapUrl *)
@@ -405,17 +406,16 @@ NS_SYNCRUNNABLEMETHOD2(ImapMailFolderSink, GetMessageId, nsIImapUrl *, nsACStrin
 
 NS_SYNCRUNNABLEMETHOD2(ImapMessageSink, SetupMsgWriteStream, nsIFile *, bool)
 NS_SYNCRUNNABLEMETHOD3(ImapMessageSink, ParseAdoptedMsgLine, const char *, nsMsgKey, nsIImapUrl *)
-NS_SYNCRUNNABLEMETHOD4(ImapMessageSink, NormalEndMsgWriteStream, nsMsgKey, bool, nsIImapUrl *, PRInt32)
+NS_SYNCRUNNABLEMETHOD4(ImapMessageSink, NormalEndMsgWriteStream, nsMsgKey, bool, nsIImapUrl *, int32_t)
 NS_SYNCRUNNABLEMETHOD0(ImapMessageSink, AbortMsgWriteStream)
 NS_SYNCRUNNABLEMETHOD0(ImapMessageSink, BeginMessageUpload)
-NS_SYNCRUNNABLEMETHOD4(ImapMessageSink, NotifyMessageFlags, PRUint32, const nsACString &, nsMsgKey, PRUint64)
+NS_SYNCRUNNABLEMETHOD4(ImapMessageSink, NotifyMessageFlags, uint32_t, const nsACString &, nsMsgKey, uint64_t)
 NS_SYNCRUNNABLEMETHOD3(ImapMessageSink, NotifyMessageDeleted, const char *, bool, const char *)
-NS_SYNCRUNNABLEMETHOD2(ImapMessageSink, GetMessageSizeFromDB, const char *, PRUint32 *)
+NS_SYNCRUNNABLEMETHOD2(ImapMessageSink, GetMessageSizeFromDB, const char *, uint32_t *)
 NS_SYNCRUNNABLEMETHOD2(ImapMessageSink, SetContentModified, nsIImapUrl *, nsImapContentModifiedType)
-NS_SYNCRUNNABLEMETHOD1(ImapMessageSink, SetImageCacheSessionForUrl, nsIMsgMailNewsUrl *)
-NS_SYNCRUNNABLEMETHOD4(ImapMessageSink, GetCurMoveCopyMessageInfo, nsIImapUrl *, PRTime *, nsACString &, PRUint32 *)
+NS_SYNCRUNNABLEMETHOD4(ImapMessageSink, GetCurMoveCopyMessageInfo, nsIImapUrl *, PRTime *, nsACString &, uint32_t *)
 
-NS_SYNCRUNNABLEMETHOD4(ImapServerSink, PossibleImapMailbox, const nsACString &, char, PRInt32, bool *)
+NS_SYNCRUNNABLEMETHOD4(ImapServerSink, PossibleImapMailbox, const nsACString &, char, int32_t, bool *)
 NS_SYNCRUNNABLEMETHOD2(ImapServerSink, FolderNeedsACLInitialized, const nsACString &, bool *)
 NS_SYNCRUNNABLEMETHOD3(ImapServerSink, AddFolderRights, const nsACString &, const nsACString &, const nsACString &)
 NS_SYNCRUNNABLEMETHOD1(ImapServerSink, RefreshFolderRights, const nsACString &)
@@ -433,10 +433,10 @@ NS_SYNCRUNNABLEMETHOD2(ImapServerSink, PrepareToRetryUrl, nsIImapUrl *, nsIImapM
 NS_SYNCRUNNABLEMETHOD1(ImapServerSink, SuspendUrl, nsIImapUrl *)
 NS_SYNCRUNNABLEMETHOD2(ImapServerSink, RetryUrl, nsIImapUrl *, nsIImapMockChannel *)
 NS_SYNCRUNNABLEMETHOD0(ImapServerSink, AbortQueuedUrls)
-NS_SYNCRUNNABLEMETHOD2(ImapServerSink, GetImapStringByID, PRInt32, nsAString &)
-NS_SYNCRUNNABLEMETHOD2(ImapServerSink, PromptLoginFailed, nsIMsgWindow *, PRInt32 *)
+NS_SYNCRUNNABLEMETHOD2(ImapServerSink, GetImapStringByName, const char*, nsAString &)
+NS_SYNCRUNNABLEMETHOD2(ImapServerSink, PromptLoginFailed, nsIMsgWindow *, int32_t *)
 NS_SYNCRUNNABLEMETHOD2(ImapServerSink, FEAlert, const nsAString &, nsIMsgMailNewsUrl *)
-NS_SYNCRUNNABLEMETHOD2(ImapServerSink, FEAlertWithID, PRInt32, nsIMsgMailNewsUrl *)
+NS_SYNCRUNNABLEMETHOD2(ImapServerSink, FEAlertWithName, const char*, nsIMsgMailNewsUrl *)
 NS_SYNCRUNNABLEMETHOD2(ImapServerSink, FEAlertFromServer, const nsACString &, nsIMsgMailNewsUrl *)
 NS_SYNCRUNNABLEMETHOD0(ImapServerSink, CommitNamespaces)
 NS_SYNCRUNNABLEMETHOD3(ImapServerSink, AsyncGetPassword, nsIImapProtocol *, bool, nsACString &)
@@ -448,3 +448,153 @@ NS_SYNCRUNNABLEMETHOD1(ImapServerSink, GetShowAttachmentsInline, bool *)
 NS_SYNCRUNNABLEMETHOD3(ImapServerSink, CramMD5Hash, const char *, const char *, char **)
 NS_SYNCRUNNABLEMETHOD1(ImapServerSink, GetLoginUsername, nsACString &)
 NS_SYNCRUNNABLEMETHOD1(ImapServerSink, UpdateTrySTARTTLSPref, bool)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, GetOriginalUsername, nsACString &)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, GetServerKey, nsACString &)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, GetServerPassword, nsACString &)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, RemoveServerConnection, nsIImapProtocol *)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, GetServerShuttingDown, bool *)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, ResetServerConnection, const nsACString &)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, SetServerDoingLsub, bool)
+NS_SYNCRUNNABLEMETHOD1(ImapServerSink, SetServerForceSelect, const nsACString &)
+
+
+namespace mozilla {
+namespace mailnews {
+
+NS_IMPL_ISUPPORTS(OAuth2ThreadHelper, msgIOAuth2ModuleListener)
+
+OAuth2ThreadHelper::OAuth2ThreadHelper(nsIMsgIncomingServer *aServer)
+: mMonitor("OAuth thread lock"),
+  mServer(aServer)
+{
+}
+
+OAuth2ThreadHelper::~OAuth2ThreadHelper()
+{
+  if (mOAuth2Support)
+  {
+    NS_ReleaseOnMainThread(mOAuth2Support.forget());
+  }
+}
+
+bool OAuth2ThreadHelper::SupportsOAuth2()
+{
+  // Acquire a lock early, before reading anything. Guarantees memory visibility
+  // issues.
+  MonitorAutoLock lockGuard(mMonitor);
+
+  // If we don't have a server, we can't init, and therefore, we don't support
+  // OAuth2.
+  if (!mServer)
+    return false;
+
+  // If we have this, then we support OAuth2.
+  if (mOAuth2Support)
+    return true;
+
+  // Initialize. This needs to be done on-main-thread: if we're off that thread,
+  // synchronously dispatch to the main thread.
+  if (NS_IsMainThread())
+  {
+    MonitorAutoUnlock lockGuard(mMonitor);
+    Init();
+  }
+  else
+  {
+    nsCOMPtr<nsIRunnable> runInit =
+      NewRunnableMethod(this, &OAuth2ThreadHelper::Init);
+    NS_DispatchToMainThread(runInit);
+    mMonitor.Wait();
+  }
+
+  // After synchronously initializing, if we didn't get an object, then we don't
+  // support XOAuth2.
+  return mOAuth2Support != nullptr;
+}
+
+void OAuth2ThreadHelper::GetXOAuth2String(nsACString &base64Str)
+{
+  MOZ_ASSERT(!NS_IsMainThread(), "This method cannot run on the main thread");
+
+  // Acquire a lock early, before reading anything. Guarantees memory visibility
+  // issues.
+  MonitorAutoLock lockGuard(mMonitor);
+
+  // Umm... what are you trying to do?
+  if (!mOAuth2Support)
+    return;
+
+  nsCOMPtr<nsIRunnable> runInit =
+    NewRunnableMethod(this, &OAuth2ThreadHelper::Connect);
+  NS_DispatchToMainThread(runInit);
+  mMonitor.Wait();
+
+  // Now we either have the string, or we failed (in which case the string is
+  // empty).
+  base64Str = mOAuth2String;
+}
+
+void OAuth2ThreadHelper::Init()
+{
+  MOZ_ASSERT(NS_IsMainThread(), "Can't touch JS off-main-thread");
+  MonitorAutoLock lockGuard(mMonitor);
+
+  // Create the OAuth2 helper module and initialize it. If the preferences are
+  // not set up on this server, we don't support OAuth2, and we nullify our
+  // members to indicate this.
+  mOAuth2Support = do_CreateInstance(MSGIOAUTH2MODULE_CONTRACTID);
+  if (mOAuth2Support)
+  {
+    bool supportsOAuth = false;
+    mOAuth2Support->InitFromMail(mServer, &supportsOAuth);
+    if (!supportsOAuth)
+      mOAuth2Support = nullptr;
+  }
+
+  // There is now no longer any need for the server. Kill it now--this helps
+  // prevent us from maintaining a refcount cycle.
+  mServer = nullptr;
+
+  // Notify anyone waiting that we're done.
+  mMonitor.Notify();
+}
+
+void OAuth2ThreadHelper::Connect()
+{
+  MOZ_ASSERT(NS_IsMainThread(), "Can't touch JS off-main-thread");
+  MOZ_ASSERT(mOAuth2Support, "Should not be here if no OAuth2 support");
+
+  // OK to delay lock since mOAuth2Support is only written on main thread.
+  nsresult rv = mOAuth2Support->Connect(true, this);
+  // If the method failed, we'll never get a callback, so notify the monitor
+  // immediately so that IMAP can react.
+  if (NS_FAILED(rv))
+  {
+    MonitorAutoLock lockGuard(mMonitor);
+    mMonitor.Notify();
+  }
+}
+
+nsresult OAuth2ThreadHelper::OnSuccess(const nsACString &aAccessToken)
+{
+  MOZ_ASSERT(NS_IsMainThread(), "Can't touch JS off-main-thread");
+  MonitorAutoLock lockGuard(mMonitor);
+
+  MOZ_ASSERT(mOAuth2Support, "Should not be here if no OAuth2 support");
+  mOAuth2Support->BuildXOAuth2String(mOAuth2String);
+  mMonitor.Notify();
+  return NS_OK;
+}
+
+nsresult OAuth2ThreadHelper::OnFailure(nsresult aError)
+{
+  MOZ_ASSERT(NS_IsMainThread(), "Can't touch JS off-main-thread");
+  MonitorAutoLock lockGuard(mMonitor);
+
+  mOAuth2String.Truncate();
+  mMonitor.Notify();
+  return NS_OK;
+}
+
+} // namespace mailnews
+} // namespace mozilla
