@@ -9,6 +9,8 @@
 
 */
 
+#include "nsOESettings.h"
+#include "mozilla/WindowsVersion.h"
 #include "nsCOMPtr.h"
 #include "nscore.h"
 #include "nsMsgUtils.h"
@@ -19,7 +21,6 @@
 #include "nsIMsgAccountManager.h"
 #include "nsIMsgAccount.h"
 #include "nsIImportSettings.h"
-#include "nsOESettings.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
 #include "nsMsgI18N.h"
@@ -48,7 +49,7 @@
 class OESettings {
 public:
   static nsresult GetDefaultMailAccount(nsAString &aMailAccount);
-  static nsresult GetCheckMailInterval(PRUint32 *aInterval);
+  static nsresult GetCheckMailInterval(uint32_t *aInterval);
   static nsresult Find50Key(nsIWindowsRegKey **aKey);
   static nsresult Find40Key(nsIWindowsRegKey **aKey);
   static nsresult FindAccountsKey(nsIWindowsRegKey **aKey);
@@ -76,19 +77,19 @@ public:
                             nsIMsgAccount *aAccount,
                             nsIWindowsRegKey *aKey,
                             const nsString &aIncomgUserName,
-                            PRInt32 authMethodIncoming, bool isNNTP);
+                            int32_t authMethodIncoming, bool isNNTP);
   static void SetSmtpServer(const nsString &aSmtpServer,
                             nsIWindowsRegKey *aKey,
                             nsIMsgIdentity *aId,
                             const nsString &aIncomgUserName,
-                            PRInt32 authMethodIncoming);
+                            int32_t authMethodIncoming);
   static nsresult GetAccountName(nsIWindowsRegKey *aKey,
                                  const nsString &aDefaultName,
                                  nsAString &aAccountName);
   static bool IsKB933612Applied();
 };
 
-static PRUint32 checkNewMailTime;// OE global setting, let's default to 30
+static uint32_t checkNewMailTime;// OE global setting, let's default to 30
 static bool     checkNewMail;    // OE global setting, let's default to false
                                  // This won't cause unwanted autodownloads-
                                  // user can set prefs after import
@@ -96,7 +97,7 @@ static bool     checkNewMail;    // OE global setting, let's default to false
 ////////////////////////////////////////////////////////////////////////
 nsresult nsOESettings::Create(nsIImportSettings** aImport)
 {
-    NS_PRECONDITION(aImport != nsnull, "null ptr");
+    NS_PRECONDITION(aImport != nullptr, "null ptr");
     if (! aImport)
         return NS_ERROR_NULL_POINTER;
 
@@ -116,19 +117,19 @@ nsOESettings::~nsOESettings()
 {
 }
 
-NS_IMPL_ISUPPORTS1(nsOESettings, nsIImportSettings)
+NS_IMPL_ISUPPORTS(nsOESettings, nsIImportSettings)
 
-NS_IMETHODIMP nsOESettings::AutoLocate(PRUnichar **description, nsIFile **location, bool *_retval)
+NS_IMETHODIMP nsOESettings::AutoLocate(char16_t **description, nsIFile **location, bool *_retval)
 {
-  NS_PRECONDITION(description != nsnull, "null ptr");
-  NS_PRECONDITION(_retval != nsnull, "null ptr");
+  NS_PRECONDITION(description != nullptr, "null ptr");
+  NS_PRECONDITION(_retval != nullptr, "null ptr");
   if (!description || !_retval)
     return NS_ERROR_NULL_POINTER;
 
   *description = nsOEStringBundle::GetStringByID(OEIMPORT_NAME);
 
   if (location)
-    *location = nsnull;
+    *location = nullptr;
 
   *_retval = false;
   nsCOMPtr<nsIWindowsRegKey> key;
@@ -149,7 +150,7 @@ NS_IMETHODIMP nsOESettings::SetLocation(nsIFile *location)
 
 NS_IMETHODIMP nsOESettings::Import(nsIMsgAccount **localMailAccount, bool *_retval)
 {
-  NS_PRECONDITION(_retval != nsnull, "null ptr");
+  NS_PRECONDITION(_retval != nullptr, "null ptr");
 
   if (OESettings::DoImport(localMailAccount)) {
     *_retval = true;
@@ -199,7 +200,7 @@ nsresult OESettings::GetDefaultMailAccount(nsAString &aMailAccount)
   return key->ReadStringValue(NS_LITERAL_STRING("Default Mail Account"), aMailAccount);
 }
 
-nsresult OESettings::GetCheckMailInterval(PRUint32 *aInterval)
+nsresult OESettings::GetCheckMailInterval(uint32_t *aInterval)
 {
   nsCOMPtr<nsIWindowsRegKey> key;
   // 'poll for messages' setting in OE is a global setting
@@ -221,7 +222,7 @@ nsresult OESettings::GetCheckMailInterval(PRUint32 *aInterval)
   if (NS_FAILED(rv))
     return rv;
 
-  PRUint32 intValue;
+  uint32_t intValue;
   rv = subKey->ReadIntValue(NS_LITERAL_STRING("Poll For Mail"), &intValue);
   if (NS_SUCCEEDED(rv) && intValue != PR_UINT32_MAX)
     *aInterval = intValue / 60000;
@@ -326,11 +327,11 @@ bool OESettings::DoImport(nsIMsgAccount **aAccount)
 
   // Iterate the accounts looking for POP3 & IMAP accounts...
   // Ignore LDAP for now!
-  PRUint32 accounts = 0;
+  uint32_t accounts = 0;
   nsAutoString keyComp;
-  PRUint32 childCount = 0;
+  uint32_t childCount = 0;
   key->GetChildCount(&childCount);
-  for (PRUint32 i = 0; i < childCount; i++) {
+  for (uint32_t i = 0; i < childCount; i++) {
     nsAutoString keyName;
     key->GetChildName(i, keyName);
 
@@ -341,12 +342,12 @@ bool OESettings::DoImport(nsIMsgAccount **aAccount)
     if (NS_FAILED(rv))
       continue;
 
-    nsCAutoString nativeKeyName;
+    nsAutoCString nativeKeyName;
     NS_CopyUnicodeToNative(keyName, nativeKeyName);
     IMPORT_LOG1("Opened Outlook Express account: %s\n",
                 nativeKeyName.get());
 
-    nsIMsgAccount  *anAccount = nsnull;
+    nsIMsgAccount  *anAccount = nullptr;
     nsAutoString value;
     rv = subKey->ReadStringValue(NS_LITERAL_STRING("IMAP Server"), value);
     if (NS_SUCCEEDED(rv) && DoIMAPServer(accMgr, subKey, value, &anAccount))
@@ -394,7 +395,7 @@ bool OESettings::DoIMAPServer(nsIMsgAccountManager *aMgr,
                               nsIMsgAccount **ppAccount)
 {
   if (ppAccount)
-    *ppAccount = nsnull;
+    *ppAccount = nullptr;
 
   nsAutoString userName;
   nsresult rv;
@@ -402,9 +403,9 @@ bool OESettings::DoIMAPServer(nsIMsgAccountManager *aMgr,
   if (NS_FAILED(rv))
     return false;
 
-  nsCAutoString nativeUserName;
+  nsAutoCString nativeUserName;
   NS_CopyUnicodeToNative(userName, nativeUserName);
-  nsCAutoString nativeServerName;
+  nsAutoCString nativeServerName;
   NS_CopyUnicodeToNative(aServerName, nativeServerName);
   // I now have a user name/server name pair, find out if it already exists?
   nsCOMPtr<nsIMsgIncomingServer> in;
@@ -422,7 +423,7 @@ bool OESettings::DoIMAPServer(nsIMsgAccountManager *aMgr,
 
     IMPORT_LOG0("Created an identity and added to existing IMAP incoming server\n");
     // Fiddle with the identities
-    PRInt32 authMethod;
+    int32_t authMethod;
     in->GetAuthMethod(&authMethod);
     SetIdentities(aMgr, account, aKey, userName, authMethod, false);
     if (ppAccount)
@@ -442,7 +443,7 @@ bool OESettings::DoIMAPServer(nsIMsgAccountManager *aMgr,
   rv = aKey->ReadStringValue(NS_LITERAL_STRING("IMAP Root Folder"), rootFolder);
   if (NS_SUCCEEDED(rv) && !rootFolder.IsEmpty()) {
     nsCOMPtr<nsIImapIncomingServer> imapServer = do_QueryInterface(in);
-    nsCAutoString nativeRootFolder;
+    nsAutoCString nativeRootFolder;
     NS_CopyUnicodeToNative(rootFolder, nativeRootFolder);
     imapServer->SetServerDirectory(nativeRootFolder);
   }
@@ -465,7 +466,7 @@ bool OESettings::DoIMAPServer(nsIMsgAccountManager *aMgr,
     IMPORT_LOG0("Created an account and set the IMAP server as the incoming server\n");
 
     // Fiddle with the identities
-    PRInt32 authMethod;
+    int32_t authMethod;
     in->GetAuthMethod(&authMethod);
     SetIdentities(aMgr, account, aKey, userName, authMethod, false);
     if (ppAccount)
@@ -482,7 +483,7 @@ bool OESettings::DoPOP3Server(nsIMsgAccountManager *aMgr,
                               nsIMsgAccount **ppAccount)
 {
   if (ppAccount)
-    *ppAccount = nsnull;
+    *ppAccount = nullptr;
 
   nsAutoString userName;
   nsresult rv;
@@ -490,9 +491,9 @@ bool OESettings::DoPOP3Server(nsIMsgAccountManager *aMgr,
   if (NS_FAILED(rv))
     return false;
 
-  nsCAutoString nativeUserName;
+  nsAutoCString nativeUserName;
   NS_CopyUnicodeToNative(userName, nativeUserName);
-  nsCAutoString nativeServerName;
+  nsAutoCString nativeServerName;
   NS_CopyUnicodeToNative(aServerName, nativeServerName);
 
   // I now have a user name/server name pair, find out if it already exists?
@@ -511,7 +512,7 @@ bool OESettings::DoPOP3Server(nsIMsgAccountManager *aMgr,
     if (NS_SUCCEEDED(rv) && account) {
       IMPORT_LOG0("Created identity and added to existing POP3 incoming server.\n");
       // Fiddle with the identities
-      PRInt32 authMethod;
+      int32_t authMethod;
       in->GetAuthMethod(&authMethod);
       SetIdentities(aMgr, account, aKey, userName, authMethod, false);
       if (ppAccount)
@@ -559,7 +560,7 @@ bool OESettings::DoPOP3Server(nsIMsgAccountManager *aMgr,
       pop3Server->SetDeferredToAccount(localFoldersAcctKey);
     }
 
-    PRUint32 intValue;
+    uint32_t intValue;
     rv = aKey->ReadIntValue(NS_LITERAL_STRING("POP3 Skip Account"), &intValue);
     // OE:0=='Include this account when receiving mail or synchronizing'==
     // TB:1==AM:Server:advanced:Include this server when getting new mail
@@ -580,7 +581,7 @@ bool OESettings::DoPOP3Server(nsIMsgAccountManager *aMgr,
     rv = aKey->ReadIntValue(NS_LITERAL_STRING("Expire Days"),
                             &intValue);
     if (NS_SUCCEEDED(rv))
-      pop3Server->SetNumDaysToLeaveOnServer(static_cast<PRInt32>(intValue));
+      pop3Server->SetNumDaysToLeaveOnServer(static_cast<int32_t>(intValue));
   }
   IMPORT_LOG2("Created POP3 server named: %s, userName: %s\n",
               nativeServerName.get(), nativeUserName.get());
@@ -595,7 +596,7 @@ bool OESettings::DoPOP3Server(nsIMsgAccountManager *aMgr,
     rv = account->SetIncomingServer(in);
     IMPORT_LOG0("Created a new account and set the incoming server to the POP3 server.\n");
 
-    PRInt32 authMethod;
+    int32_t authMethod;
     in->GetAuthMethod(&authMethod);
     // Fiddle with the identities
     SetIdentities(aMgr, account, aKey, userName, authMethod, false);
@@ -614,7 +615,7 @@ bool OESettings::DoNNTPServer(nsIMsgAccountManager *aMgr,
                               nsIMsgAccount **ppAccount)
 {
   if (ppAccount)
-    *ppAccount = nsnull;
+    *ppAccount = nullptr;
 
   nsAutoString userName;
   nsresult rv;
@@ -623,7 +624,7 @@ bool OESettings::DoNNTPServer(nsIMsgAccountManager *aMgr,
 
   bool result = false;
 
-  nsCAutoString nativeServerName;
+  nsAutoCString nativeServerName;
   NS_CopyUnicodeToNative(aServerName, nativeServerName);
   // I now have a user name/server name pair, find out if it already exists?
   // NNTP can have empty user name.  This is wild card in findserver
@@ -632,20 +633,20 @@ bool OESettings::DoNNTPServer(nsIMsgAccountManager *aMgr,
                         nativeServerName,
                         NS_LITERAL_CSTRING("nntp"),
                         getter_AddRefs(in));
-  if (NS_FAILED(rv) || (in == nsnull)) {
+  if (NS_FAILED(rv) || (in == nullptr)) {
     // Create the incoming server and an account for it?
     rv = aMgr->CreateIncomingServer(EmptyCString(),
                                     nativeServerName,
                                     NS_LITERAL_CSTRING("nntp"),
                                     getter_AddRefs(in));
     if (NS_SUCCEEDED(rv) && in) {
-      PRUint32 port = 0;
+      uint32_t port = 0;
       rv = aKey->ReadIntValue(NS_LITERAL_STRING("NNTP Port"),
                               &port);
       if (NS_SUCCEEDED(rv) && port && port != 119)
-        in->SetPort(static_cast<PRInt32>(port));
+        in->SetPort(static_cast<int32_t>(port));
 
-      nsCAutoString nativeUserName;
+      nsAutoCString nativeUserName;
       NS_CopyUnicodeToNative(userName, nativeUserName);
       // do nntpincomingserver stuff
       nsCOMPtr<nsINntpIncomingServer> nntpServer = do_QueryInterface(in);
@@ -702,22 +703,22 @@ void OESettings::SetIncomingServerProperties(nsIMsgIncomingServer *aServer,
                                              const nsString &aKeyNamePrefix)
 {
   nsresult rv;
-  PRUint32 secureConnection = 0;
+  uint32_t secureConnection = 0;
   nsString keyName(aKeyNamePrefix);
   keyName.AppendLiteral("Secure Connection");
   rv = aKey->ReadIntValue(keyName, &secureConnection);
   if (NS_SUCCEEDED(rv) && secureConnection == 1)
     aServer->SetSocketType(nsMsgSocketType::SSL);
 
-  PRUint32 port = 0;
+  uint32_t port = 0;
   keyName.SetLength(aKeyNamePrefix.Length());
   keyName.AppendLiteral("Port");
   rv = aKey->ReadIntValue(keyName, &port);
   if (NS_SUCCEEDED(rv) && port)
-    aServer->SetPort(static_cast<PRInt32>(port));
+    aServer->SetPort(static_cast<int32_t>(port));
 
-  PRInt32 authMethod;
-  PRUint32 useSicily = 0;
+  int32_t authMethod;
+  uint32_t useSicily = 0;
   keyName.SetLength(aKeyNamePrefix.Length());
   keyName.AppendLiteral("Use Sicily");
   rv = aKey->ReadIntValue(keyName, &useSicily);
@@ -735,7 +736,7 @@ void OESettings::SetIdentities(nsIMsgAccountManager *aMgr,
                                nsIMsgAccount *pAcc,
                                nsIWindowsRegKey *aKey,
                                const nsString &aIncomgUserName,
-                               PRInt32 authMethodIncoming,
+                               int32_t authMethodIncoming,
                                bool isNNTP)
 {
   // Get the relevant information for an identity
@@ -767,15 +768,13 @@ void OESettings::SetIdentities(nsIMsgAccountManager *aMgr,
     return;
 
   id->SetFullName(name);
-  //BUG 470587. Don't set this: id->SetIdentityName(fullName);
-
   id->SetOrganization(orgName);
 
-  nsCAutoString nativeEmail;
+  nsAutoCString nativeEmail;
   NS_CopyUnicodeToNative(email, nativeEmail);
   id->SetEmail(nativeEmail);
   if (!reply.IsEmpty()) {
-    nsCAutoString nativeReply;
+    nsAutoCString nativeReply;
     NS_CopyUnicodeToNative(reply, nativeReply);
     id->SetReplyTo(nativeReply);
   }
@@ -784,7 +783,7 @@ void OESettings::SetIdentities(nsIMsgAccountManager *aMgr,
   id->SetReplyOnTop(isNNTP ? 0 : 1);
   pAcc->AddIdentity(id);
 
-  nsCAutoString nativeName;
+  nsAutoCString nativeName;
   NS_CopyUnicodeToNative(name, nativeName);
   IMPORT_LOG0("Created identity and added to the account\n");
   IMPORT_LOG1("\tname: %s\n", nativeName.get());
@@ -801,7 +800,7 @@ void OESettings::SetSmtpServer(const nsString &aSmtpServer,
                                nsIWindowsRegKey *aKey,
                                nsIMsgIdentity *aId,
                                const nsString &aIncomgUserName,
-                               PRInt32 authMethodIncoming)
+                               int32_t authMethodIncoming)
 {
   // set the id.smtpserver accordingly
   // first we have to calculate the smtp user name which is based on sicily
@@ -809,7 +808,7 @@ void OESettings::SetSmtpServer(const nsString &aSmtpServer,
     return;
   nsCString smtpServerKey;
   // smtp user name depends on sicily which may or not exist
-  PRUint32 useSicily = 0;
+  uint32_t useSicily = 0;
   nsresult rv = aKey->ReadIntValue(NS_LITERAL_STRING("SMTP Use Sicily"),
                                    &useSicily);
   nsAutoString userName;
@@ -832,9 +831,9 @@ void OESettings::SetSmtpServer(const nsString &aSmtpServer,
     nsCOMPtr<nsISmtpServer> foundServer;
     // don't try to make another server
     // regardless if username doesn't match
-    nsCAutoString nativeUserName;
+    nsAutoCString nativeUserName;
     NS_CopyUnicodeToNative(userName, nativeUserName);
-    nsCAutoString nativeSmtpServer;
+    nsAutoCString nativeSmtpServer;
     NS_CopyUnicodeToNative(aSmtpServer, nativeSmtpServer);
     rv = smtpService->FindServer(nativeUserName.get(),
                                  nativeSmtpServer.get(),
@@ -849,16 +848,16 @@ void OESettings::SetSmtpServer(const nsString &aSmtpServer,
     }
     else {
       nsCOMPtr<nsISmtpServer> smtpServer;
-      rv = smtpService->CreateSmtpServer(getter_AddRefs(smtpServer));
+      rv = smtpService->CreateServer(getter_AddRefs(smtpServer));
       if (NS_SUCCEEDED(rv) && smtpServer) {
-        PRUint32 port = 0;
+        uint32_t port = 0;
         rv = aKey->ReadIntValue(NS_LITERAL_STRING("SMTP Port"),
                                 &port);
         if (NS_SUCCEEDED(rv) && port)
-          smtpServer->SetPort(static_cast<PRInt32>(port));
+          smtpServer->SetPort(static_cast<int32_t>(port));
 
-        PRInt32 socketType = nsMsgSocketType::plain;
-        PRUint32 secureConnection = 0;
+        int32_t socketType = nsMsgSocketType::plain;
+        uint32_t secureConnection = 0;
         rv = aKey->ReadIntValue(NS_LITERAL_STRING("SMTP Secure Connection"),
                                 &secureConnection);
         if (NS_SUCCEEDED(rv) && secureConnection == 1) {
@@ -898,27 +897,25 @@ void OESettings::SetSmtpServer(const nsString &aSmtpServer,
 
 bool OESettings::IsKB933612Applied()
 {
-  OSVERSIONINFOEX versionInfo = { 0 };
-  versionInfo.dwOSVersionInfoSize = sizeof(versionInfo);
-  GetVersionEx(reinterpret_cast<OSVERSIONINFO*>(&versionInfo));
-
-  // Windows XP SP3 and Windows Vista SP1 include KB933612 fix.
+  // The following versions of Windows include KB933612 fix:
+  // - Windows 7 and future versions of Windows
+  // - Windows Vista, SP1 or later
+  // - Windows Server 2003, SP2 or later
+  // - Windows XP, SP3 or later
+  //
+  // The following versions do not:
+  // - Windows Vista SP0
+  // - Windows Server 2003, SP1 or earlier
+  // - Windows XP, SP2 or earlier
+  //
   // See http://support.microsoft.com/kb/929123 and
   // http://support.microsoft.com/kb/933612
-  switch (versionInfo.dwMajorVersion) {
-    case 6:
-      if ((versionInfo.dwMinorVersion == 0 && versionInfo.wServicePackMajor > 0) ||
-          versionInfo.dwMinorVersion == 1) {
-        return true;
-      }
-      break;
-    case 5:
-      if (versionInfo.dwMinorVersion > 0 && versionInfo.wServicePackMajor > 2)
-        return true;
-      break;
-    default:
-      break;
-  }
-  return false;
+  //
+  // Note that mozilla::IsWin2003SP2OrLater() will return true for
+  // Windows Vista and mozilla::IsXPSP3OrLater() will return true
+  // for Windows Server 2003.
+  return mozilla::IsVistaSP1OrLater() ||
+         !mozilla::IsWin2003OrLater() && mozilla::IsXPSP3OrLater() ||
+         !mozilla::IsVistaOrLater() && mozilla::IsWin2003SP2OrLater();
 }
 
