@@ -5,7 +5,7 @@
 
 const kNonVcardFields =
         ["NickNameContainer", "SecondaryEmailContainer", "ScreenNameContainer",
-         "customFields", "allowRemoteContent", "preferDisplayName"];
+         "customFields", "preferDisplayName"];
 
 const kPhoneticFields =
         ["PhoneticLastName", "PhoneticLabel1", "PhoneticSpacer1",
@@ -55,7 +55,6 @@ const kVcardFields =
           // Other > Notes
          ["Notes", "Notes"]];
 
-const kDefaultYear = 2000;
 var gEditCard;
 var gOnSaveListeners = [];
 var gOnLoadListeners = [];
@@ -80,7 +79,8 @@ function OnLoadNewCard()
   {
     gEditCard.selectedAB = kPersonalAddressbookURI;
 
-    if ("selectedAB" in window.arguments[0]) {
+    if ("selectedAB" in window.arguments[0] &&
+        (window.arguments[0].selectedAB != kAllDirectoryRoot + "?")) {
       // check if selected ab is a mailing list
       var abURI = window.arguments[0].selectedAB;
 
@@ -110,10 +110,6 @@ function OnLoadNewCard()
       gEditCard.card.setProperty("_AimScreenName",
                                  window.arguments[0].aimScreenName);
 
-    if ("allowRemoteContent" in window.arguments[0])
-      gEditCard.card.setProperty("AllowRemoteContent",
-                                 window.arguments[0].allowRemoteContent);
-
     if ("okCallback" in window.arguments[0])
       gOkCallback = window.arguments[0].okCallback;
 
@@ -137,7 +133,7 @@ function OnLoadNewCard()
   if (gHideABPicker && abPopup) {
     abPopup.hidden = true;
     document.getElementById("abPopupLabel").hidden = true;
-  } 
+  }
 
   SetCardDialogTitle(gEditCard.card.displayName);
 
@@ -192,11 +188,11 @@ function EditCardOKButton()
       }
     }
   }
- 
+
   CheckAndSetCardValues(gEditCard.card, document, false);
 
   directory.modifyCard(gEditCard.card);
- 
+
   while (foundDirectories.length)
   {
     // Update the addressLists item for this card
@@ -251,7 +247,7 @@ function OnLoadEditCard()
       var abURI = window.arguments[0].abURI;
       var directory = GetDirectoryFromURI(abURI);
 
-      if (directory.readOnly) 
+      if (directory.readOnly)
       {
         // Set all the editable vcard fields to read only
         for (var i = kVcardFields.length; i-- > 0; )
@@ -274,9 +270,8 @@ function OnLoadEditCard()
         document.getElementById(kPhoneticFields[0]).readOnly = true;
         document.getElementById(kPhoneticFields[3]).readOnly = true;
 
-        // Also disable the mail format popup and allow remote content items.
+        // Also disable the mail format popup.
         document.getElementById("PreferMailFormatPopup").disabled = true;
-        document.getElementById("allowRemoteContent").disabled = true;
 
         // And the "prefer display name" checkbox.
         document.getElementById("preferDisplayName").disabled = true;
@@ -284,10 +279,6 @@ function OnLoadEditCard()
         document.documentElement.buttons = "accept";
         document.documentElement.removeAttribute("ondialogaccept");
       }
-
-      // hide remote content in HTML field for remote directories
-      if (directory.isRemote)
-        document.getElementById('allowRemoteContent').hidden = true;
     }
   }
 }
@@ -353,10 +344,10 @@ function NotifySaveListeners(directory)
 function InitPhoneticFields()
 {
   var showPhoneticFields =
-    Services.prefs.getComplexValue("mail.addr_book.show_phonetic_fields", 
+    Services.prefs.getComplexValue("mail.addr_book.show_phonetic_fields",
       Components.interfaces.nsIPrefLocalizedString).data;
 
-  // hide phonetic fields if indicated by the pref
+  // show phonetic fields if indicated by the pref
   if (showPhoneticFields == "true")
   {
     for (var i = kPhoneticFields.length; i-- > 0; )
@@ -376,7 +367,7 @@ function InitEditCard()
   // get specific prefs that gEditCard will need
   try {
     var displayLastNameFirst =
-      Services.prefs.getComplexValue("mail.addr_book.displayName.lastnamefirst", 
+      Services.prefs.getComplexValue("mail.addr_book.displayName.lastnamefirst",
         Components.interfaces.nsIPrefLocalizedString).data;
     gEditCard.displayLastNameFirst = (displayLastNameFirst == "true");
     gEditCard.generateDisplayName =
@@ -420,11 +411,6 @@ function NewCardOKButton()
       var directory = GetDirectoryFromURI(uri);
       gEditCard.card = directory.addCard(gEditCard.card);
       NotifySaveListeners(directory);
-      if ("arguments" in window && window.arguments[0] &&
-          "allowRemoteContent" in window.arguments[0])
-        // getProperty may return a "1" or "0" string, we want a boolean
-        window.arguments[0].allowRemoteContent =
-          gEditCard.card.getProperty("AllowRemoteContent", false) != false;
     }
   }
 
@@ -449,6 +435,15 @@ function GetCardValues(cardproperty, doc)
   var birthday = doc.getElementById("Birthday");
   modifyDatepicker(birthday);
 
+  // Get the year first, so that the following month/day
+  // calculations can take leap years into account.
+  var year = cardproperty.getProperty("BirthYear", null);
+  var birthYear = doc.getElementById("BirthYear");
+  // set the year in the datepicker to the stored year
+  // if the year isn't present, default to 2000 (a leap year)
+  birthday.year = saneBirthYear(year);
+  birthYear.value = year;
+
   // get the month of the year (1 - 12)
   var month = cardproperty.getProperty("BirthMonth", null);
   if (month > 0 && month < 13)
@@ -463,14 +458,6 @@ function GetCardValues(cardproperty, doc)
   else
     birthday.dateField.value = null;
 
-  // get the year
-  var year = cardproperty.getProperty("BirthYear", null);
-  var birthYear = doc.getElementById("BirthYear");
-  // set the year in the datepicker to the stored year
-  // if the year isn't present, default to 2000 (a leap year)
-  birthday.year = year && year < 10000 && year > 0 ? year : kDefaultYear;
-  birthYear.value = year;
-
   // get the current age
   calculateAge(null, birthYear);
   // when the birth year changes, update the datepicker's year to the new value
@@ -484,11 +471,6 @@ function GetCardValues(cardproperty, doc)
   if (popup)
     popup.value = cardproperty.getProperty("PreferMailFormat", "");
 
-  var allowRemoteContentEl = document.getElementById("allowRemoteContent");
-  if (allowRemoteContentEl)
-    // getProperty may return a "1" or "0" string, we want a boolean
-    allowRemoteContentEl.checked = cardproperty.getProperty("AllowRemoteContent", false) != false;
-
   var preferDisplayNameEl = document.getElementById("preferDisplayName");
   if (preferDisplayNameEl)
     // getProperty may return a "1" or "0" string, we want a boolean
@@ -555,10 +537,6 @@ function CheckAndSetCardValues(cardproperty, doc, check)
   if (popup)
     cardproperty.setProperty("PreferMailFormat", popup.value);
 
-  var allowRemoteContentEl = document.getElementById("allowRemoteContent");
-  if (allowRemoteContentEl)
-    cardproperty.setProperty("AllowRemoteContent", allowRemoteContentEl.checked);
-
   var preferDisplayNameEl = document.getElementById("preferDisplayName");
   if (preferDisplayNameEl)
     cardproperty.setProperty("PreferDisplayName", preferDisplayNameEl.checked);
@@ -693,7 +671,7 @@ function calculateAge(aEvent, aElement) {
     yearElem.value = datepicker.year;
   var year = yearElem.value;
   // if the year element's value is invalid set the year and age elements to null
-  if (isNaN(year) || year < 1 || year > 9999) {
+  if (isNaN(year) || year < kMinYear || year > kMaxYear) {
     yearElem.value = null;
     ageElem.value = null;
     datepicker.year = kDefaultYear;
@@ -732,10 +710,10 @@ function calculateYear(aEvent, aElement) {
   if (aEvent)
     aElement = this;
   if (aElement.id == "Age") {
-    yearElem = document.getElementById("BirthYear");
     datepicker = document.getElementById("Birthday");
+    yearElem = document.getElementById("BirthYear");
   }
-  if (!yearElem || !datepicker)
+  if (!datepicker || !yearElem)
     return;
 
   // if the age is null, remove the year from the year element, and set the
@@ -759,7 +737,9 @@ function calculateYear(aEvent, aElement) {
     datepicker.year = kDefaultYear;
     // if there was an error (like invalid year) set the year and age to null
     yearElem.value = null;
-    ageElem.value = null;
+    let ageElem = document.getElementById("Age");
+    if (ageElem)
+      ageElem.value = null;
   }
 }
 
@@ -774,7 +754,7 @@ function calculateYear(aEvent, aElement) {
  * If any field is blank, the corresponding property is either the previous
  * value if there was one since the card was opened or the relevant portion of
  * the current date.
- * 
+ *
  * To get the displayed values, get the value of the individual field, such as
  * datepicker.yyyyField.value where yyyy is "year", "month", or "date" for the
  * year, month, and day, respectively.
@@ -809,7 +789,7 @@ function modifyDatepicker(aDatepicker) {
       var dt = new Date(this.year, currentMonth, aValue);
       return dt.getMonth() != currentMonth ? 1 : aValue;
     }
-    var max = (aField == this.monthField) ? 11 : 9999;
+    var max = (aField == this.monthField) ? 11 : kMaxYear;
     // make sure the value isn't too high
     if (aValue > max)
       return aNoWrap ? max : min;
@@ -818,7 +798,7 @@ function modifyDatepicker(aDatepicker) {
   // sets the specified field to the given value, but allows blank fields
   // from: mozilla/toolkit/content/widgets/datetimepicker.xml#698
   aDatepicker._setFieldValue = function setValue(aField, aValue) {
-    if (aField == this.yearField && aValue > 0 && aValue < 10000) {
+    if (aField == this.yearField && aValue >= kMinYear && aValue <= kMaxYear) {
       var oldDate = this._dateValue;
       this._dateValue.setFullYear(aValue);
       if (oldDate != this._dateValue) {
@@ -962,7 +942,7 @@ function browsePhoto() {
   var fp = Components.classes["@mozilla.org/filepicker;1"]
 	                   .createInstance(nsIFilePicker);
   fp.init(window, gAddressBookBundle.getString("browsePhoto"), nsIFilePicker.modeOpen);
-  
+
   // Add All Files & Image Files filters and select the latter
   fp.appendFilters(nsIFilePicker.filterImages);
   fp.appendFilters(nsIFilePicker.filterAll);
@@ -1073,6 +1053,10 @@ var gFilePhotoHandler =
     if (!file)
       return false;
 
+    // If the local file has been removed/renamed, keep the current photo as is.
+    if (!file.exists() || !file.isFile())
+      return true;
+
     var photoURI = Services.io.newFileURI(file).spec;
 
     var file = storePhoto(photoURI);
