@@ -676,6 +676,28 @@ function OnLoadMessenger()
   verifyAccounts(null, false);
   msgDBCacheManager.init();
 
+  // set the messenger default window size relative to the screen size
+  // initial default dimensions are 2/3 and 1/2 of the screen dimensions
+  if (!document.documentElement.hasAttribute("width")) {
+    let screenHeight  = window.screen.availHeight;
+    let screenWidth   = window.screen.availWidth;
+    let defaultHeight = Math.floor(screenHeight * 2 / 3);
+    let defaultWidth  = Math.floor(screenWidth / 2);
+
+    // minimum dimensions are 1024x768 less padding unless restrained by screen
+    const minHeight = 768;
+    const minWidth = 1024;
+
+    if (defaultHeight < minHeight)
+       defaultHeight = Math.min(minHeight, screenHeight);
+    if (defaultWidth < minWidth)
+       defaultWidth = Math.min(minWidth, screenWidth);
+
+    // keep some distance to the borders, accounting for window decoration
+    document.documentElement.setAttribute("height", defaultHeight - 48);
+    document.documentElement.setAttribute("width",  defaultWidth  - 24);
+  }
+
   // initialize tabmail system - see tabmail.js and tabmail.xml for details
   let tabmail = GetTabMail();
   tabmail.registerTabType(gMailNewsTabsType);
@@ -1006,22 +1028,8 @@ function OnLoadFolderPane()
     folderTree.addEventListener("mousedown",TreeOnMouseDown,true);
 }
 
-function UpgradeThreadPaneUI()
-{
-  try
-  {
-    if (Services.prefs.getIntPref("mailnews.ui.threadpane.version") < 6)
-      Services.prefs.setIntPref("mailnews.ui.threadpane.version", 6);
-  }
-  catch (ex) 
-  {
-    dump("UpgradeThreadPane: ex = " + ex + "\n");
-  }
-}
-
 function OnLoadThreadPane()
 {
-  UpgradeThreadPaneUI();
   AddMutationObserver(UpdateAttachmentCol);
 }
 
@@ -1231,9 +1239,8 @@ function FolderPaneOnClick(event)
     return;
 
   var folderTree = GetFolderTree();
-  var row = {}, col = {}, elt = {};
-  folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, elt);
-  if (row.value == -1)
+  var cell = folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY);
+  if (cell.row == -1)
   {
     if (event.originalTarget.localName == "treecol")
     {
@@ -1242,10 +1249,10 @@ function FolderPaneOnClick(event)
     }
   }
   else if ((event.detail == 2) &&
-           (elt.value != "twisty") &&
-           (folderTree.view.getLevel(row.value) != 0))
+           (cell.childElt != "twisty") &&
+           (folderTree.view.getLevel(cell.row) != 0))
   {
-    FolderPaneDoubleClick(row.value, event);
+    FolderPaneDoubleClick(cell.row, event);
   }
 }
 
